#-----------------------------------------------------------------------
# Setup path and load IxLoad package.
#-----------------------------------------------------------------------
source ../setup_simple.tcl


#-----------------------------------------------------------------------
# Initialize IxLoad
# IxLoad connect should always be called, even for local scripts.
#-------------------------------------------------------------------
::IxLoad connect $::IxLoadPrivate::SimpleSettings::remoteServer


# Once we've connected, make sure we disconnect, even if there's a problem
if [catch {


#-----------------------------------------------------------------------
# Loads plugins for specific protocols configured in this test.
#-----------------------------------------------------------------------
global ixAppPluginManager
$ixAppPluginManager load "FTP"
$ixAppPluginManager load "POP3"


#-----------------------------------------------------------------------
# Logger setup.
#-----------------------------------------------------------------------
set logtag "IxLoad-api"
set logName "FTP_POP3_SM"
set logger [::IxLoad new ixLogger $logtag 1]
set logEngine [$logger getEngine]
$logEngine setLevels $::ixLogger(kLevelDebug) $::ixLogger(kLevelInfo)
$logEngine setFile $logName 2 256 1


#-----------------------------------------------------------------------
# Build Chassis Chain.
#-----------------------------------------------------------------------
set chassisChain [::IxLoad new ixChassisChain]

$chassisChain addChassis $::IxLoadPrivate::SimpleSettings::chassisName


#-----------------------------------------------------------------------
# Build Client Network.
#-----------------------------------------------------------------------
set clientNetwork [::IxLoad new ixNetworkGroup $chassisChain]

$clientNetwork config \
	-name                 	"ClientNetwork"

$clientNetwork globalPlugins.clear

set Filter_1 [::IxLoad new ixNetFilterPlugin]

$clientNetwork globalPlugins.appendItem -object $Filter_1

$Filter_1 config \
	-name                   "Filter_1"

set GratARP_1 [::IxLoad new ixNetGratArpPlugin]

$clientNetwork globalPlugins.appendItem -object $GratARP_1

$GratARP_1 config \
	-name					"GratARP_1"

set TCP_1 [::IxLoad new ixNetTCPPlugin]

$clientNetwork globalPlugins.appendItem -object $TCP_1

$TCP_1 config \
	-tcp_tw_recycle			true \
	-tcp_keepalive_time		75 \
	-tcp_keepalive_intvl	7200 \
	-tcp_wmem_default		4096 \
	-tcp_port_min			1024 \
	-tcp_port_max			65535 \
	-tcp_window_scaling		false \
	-name					"TCP_1" \
	-tcp_rmem_default		4096

set DNS_1 [::IxLoad new ixNetDnsPlugin]

$clientNetwork globalPlugins.appendItem -object $DNS_1

$DNS_1 config \
	-name                   "DNS_1"

$DNS_1 hostList.clear

$DNS_1 searchList.clear

$DNS_1 nameServerList.clear

set Settings_1 [::IxLoad new ixNetIxLoadSettingsPlugin]

$clientNetwork globalPlugins.appendItem -object $Settings_1

$Settings_1 config \
	-name					"Settings_1"

set Ethernet_1 [$clientNetwork getL1Plugin]

set my_ixNetEthernetELMPlugin_1 [::IxLoad new ixNetEthernetELMPlugin]

$Ethernet_1 config \
	-name					"Ethernet /PHY_1" \
	-cardElm				$my_ixNetEthernetELMPlugin_1

$Ethernet_1 childrenList.clear

set MAC_VLAN_1 [::IxLoad new ixNetL2EthernetPlugin]

$Ethernet_1 childrenList.appendItem -object $MAC_VLAN_1

$MAC_VLAN_1 config \
	-name                   "MAC/VLAN_1"

$MAC_VLAN_1 childrenList.clear

set IP_1 [::IxLoad new ixNetIpV4V6Plugin]

$MAC_VLAN_1 childrenList.appendItem -object $IP_1

$IP_1 config \
	-name					"IP_1"

$IP_1 childrenList.clear

$IP_1 extensionList.clear

$MAC_VLAN_1 extensionList.clear

$Ethernet_1 extensionList.clear


#-----------------------------------------------------------------------
# Setting the ranges starting with the plugin on top of the stack.
#-----------------------------------------------------------------------
$IP_1 rangeList.clear

set IP_R1 [::IxLoad new ixNetIpV4V6Range]

$IP_1 rangeList.appendItem -object $IP_R1

$IP_R1 config \
	-count                  10 \
	-name                   "IP_R1" \
	-gatewayAddress         "0.0.0.0" \
	-prefix                 16 \
	-mss                    100 \
	-ipAddress              "198.18.0.1"

set MAC_R1 [$IP_R1 getLowerRelatedRange "MacRange"]

$MAC_R1 config \
	-count                  10 \
	-mac                    "00:C6:12:02:01:00" \
	-name                   "MAC_R1"

set VLAN_R1 [$IP_R1 getLowerRelatedRange "VlanIdRange"]

$VLAN_R1 config \
	-name                   "VLAN_R1" \
	-idIncrMode             1 \
	-priority               0 \
	-innerPriority          0


#-----------------------------------------------------------------------
# Assign Ports to Client Network.
#-----------------------------------------------------------------------
$clientNetwork portList.appendItem \
    -chassisId  			1 \
    -cardId     			$::IxLoadPrivate::SimpleSettings::clientPort(CARD_ID)\
    -portId     			$::IxLoadPrivate::SimpleSettings::clientPort(PORT_ID)


#-----------------------------------------------------------------------
# Build Server Network.
#-----------------------------------------------------------------------
set serverNetwork [::IxLoad new ixNetworkGroup $chassisChain]

$serverNetwork config \
	-name                   "ServerNetwork"

$serverNetwork globalPlugins.clear

set Filter_2 [::IxLoad new ixNetFilterPlugin]

$serverNetwork globalPlugins.appendItem -object $Filter_2

$Filter_2 config \
	-name                  	"Filter_2"

set GratARP_2 [::IxLoad new ixNetGratArpPlugin]

$serverNetwork globalPlugins.appendItem -object $GratARP_2

$GratARP_2 config \
	-name                   "GratARP_2"

set TCP_2 [::IxLoad new ixNetTCPPlugin]

$serverNetwork globalPlugins.appendItem -object $TCP_2

$TCP_2 config \
	-tcp_tw_recycle			true \
	-tcp_keepalive_time     75 \
	-tcp_keepalive_intvl    7200 \
	-tcp_wmem_default       4096 \
	-tcp_port_min           1024 \
	-tcp_port_max           65535 \
	-tcp_window_scaling     false \
	-name                   "TCP_2" \
	-tcp_rmem_default       4096

set DNS_2 [::IxLoad new ixNetDnsPlugin]

$serverNetwork globalPlugins.appendItem -object $DNS_2

$DNS_2 config \
	-name					"DNS_2"

$DNS_2 hostList.clear

$DNS_2 searchList.clear

$DNS_2 nameServerList.clear

set Settings_2 [::IxLoad new ixNetIxLoadSettingsPlugin]

$serverNetwork globalPlugins.appendItem -object $Settings_2

$Settings_2 config \
	-name					"Settings_2"

set Ethernet_2 [$serverNetwork getL1Plugin]

set my_ixNetEthernetELMPlugin_2 [::IxLoad new ixNetEthernetELMPlugin]

$Ethernet_2 config \
	-name					"Ethernet /PHY_2" \
	-cardElm                $my_ixNetEthernetELMPlugin_2

$Ethernet_2 childrenList.clear

set MAC_VLAN_2 [::IxLoad new ixNetL2EthernetPlugin]

$Ethernet_2 childrenList.appendItem -object $MAC_VLAN_2

$MAC_VLAN_2 config \
	-name					"MAC/VLAN_2"

$MAC_VLAN_2 childrenList.clear

set IP_2 [::IxLoad new ixNetIpV4V6Plugin]

$MAC_VLAN_2 childrenList.appendItem -object $IP_2

$IP_2 config \
	-name                   "IP_2"

$IP_2 childrenList.clear

$IP_2 extensionList.clear

$MAC_VLAN_2 extensionList.clear

$Ethernet_2 extensionList.clear


#-----------------------------------------------------------------------
# Setting the ranges starting with the plugin on top of the stack.
#-----------------------------------------------------------------------
$IP_2 rangeList.clear

set IP_R2 [::IxLoad new ixNetIpV4V6Range]

$IP_2 rangeList.appendItem -object $IP_R2

$IP_R2 config \
	-count                  10 \
	-name                   "IP_R2" \
	-gatewayAddress         "0.0.0.0" \
	-prefix                 16 \
	-ipAddress              "198.18.0.101"

set MAC_R2 [$IP_R2 getLowerRelatedRange "MacRange"]

$MAC_R2 config \
	-count                  10 \
	-mac                    "00:C6:12:02:02:00" \
	-name                   "MAC_R2"

set VLAN_R2 [$IP_R2 getLowerRelatedRange "VlanIdRange"]

$VLAN_R2 config \
	-name                   "VLAN_R2" \
	-idIncrMode             1 \
	-priority               0 \
	-innerPriority          0


#-----------------------------------------------------------------------
# Assign Ports to Server Network.
#-----------------------------------------------------------------------
$serverNetwork portList.appendItem \
    -chassisId  			1 \
    -cardId     			$::IxLoadPrivate::SimpleSettings::serverPort(CARD_ID) \
    -portId     			$::IxLoadPrivate::SimpleSettings::serverPort(PORT_ID)


#-----------------------------------------------------------------------
# FTP_POP3 Client Configuration.
#-----------------------------------------------------------------------
set clientTraffic [::IxLoad new ixClientTraffic -name "ClientTraffic"]

$clientTraffic agentList.appendItem \
    -name       			"FTPClient" \
    -protocol   			"FTP" \
    -type       			"Client" \
    -userName   			"root" \
    -password   			"noreply@ixiacom.com" \
    -fileList   			{'/#64', '/#256', '/#1024', '/#4096'} \
    -mode       			$::FTP_Client(kModeActive) \
    -enableEsm  			0 \
    -esm        			300        

$clientTraffic agentList(0).actionList.clear

$clientTraffic agentList(0).actionList.appendItem \
    -command    			"GET" \
    -destination 			"ServerTraffic_FTPServer" \
    -userName    			"root" \
    -password    			"noreply@ixiacom.com" \
    -arguments   			"/#4096"

$clientTraffic agentList.appendItem \
    -name               	"POP3Client" \
    -protocol           	"POP3" \
    -type               	"Client" \
    -commandTimeout     	120

$clientTraffic agentList(1).commandList.clear

$clientTraffic agentList(1).commandList.appendItem \
    -command    			$::Pop3Command(kUser) \
    -arguments  			$::Pop3Command(kDefaultUserName)

$clientTraffic agentList(1).commandList.appendItem \
    -command    			$::Pop3Command(kPassword) \
    -arguments  			$::Pop3Command(kDefaultPassword)

$clientTraffic agentList(1).commandList.appendItem \
    -command    			$::Pop3Command(kOpen) \
    -arguments  			"198.18.0.101" 
   

#-----------------------------------------------------------------------
# FTP_POP3 Server Configuration.
#-----------------------------------------------------------------------
set serverTraffic [::IxLoad new ixServerTraffic -name "ServerTraffic"]

$serverTraffic agentList.appendItem \
    -name       			"FTPServer" \
    -protocol   			"FTP" \
    -type       			"Server" \
    -ftpPort    			21 \
    -enableEsm  			0 \
    -esm        			300

$serverTraffic agentList.appendItem \
    -name                   "POP3Server" \
    -protocol               "POP3" \
    -type                   "Server" \
    -concurrentSessionLimit $::POP3_Server(kConcurrentSessionLimitDefault)

$serverTraffic agentList(1).mailMessageList.clear

$serverTraffic agentList(1).mailMessageList.appendItem \
    -name           		"my_simple_mail_message" \
    -description    		"100 bytes plain text body" \
    -bodyFormat     		$::MailMessage(kBodyFormatPlainText) \
    -bodySizeType   		$::MailMessage(kBodySizeTypeFixed) \
    -bodySizeFixed  		100

set From7 [::IxLoad new MailHeader]

$From7 config \
	-isCustomHdr            false \
	-name                   "From" \
	-value                  "fromName@company.com" 
	
$serverTraffic agentList(1).mailMessageList(0).headerList.appendItem -object $From7

set To7 [::IxLoad new MailHeader]

$To7 config \
	-isCustomHdr            false \
	-name                   "To" \
	-value                  "toName@company.com" 

$serverTraffic agentList(1).mailMessageList(0).headerList.appendItem -object $To7

set Subject7 [::IxLoad new MailHeader]

$Subject7 config \
	-isCustomHdr            true \
	-name                   "Subject" \
	-value                  "sample subject" 

$serverTraffic agentList(1).mailMessageList(0).headerList.appendItem -object $Subject7

$serverTraffic agentList(1).mailMessageList(0).attachmentList.appendItem \
    -type       			$::MailAttachment(kGeneratedData) \
    -dataType   			$::MailAttachment(kPlainText) \
    -sizeMin    			1024 \
    -sizeMax    			4096 \
    -countMin   			1 \
    -countMax   			3 \
    -fileName   			"attachment.txt"


#-----------------------------------------------------------------------
# Create a client and server mapping and bind into the
# network and traffic that they will be employing.
#-----------------------------------------------------------------------
set clientTrafficNetworkMapping [::IxLoad new ixClientTrafficNetworkMapping \
    -network                $clientNetwork \
    -traffic                $clientTraffic \
    -objectiveType          $::ixObjective(kObjectiveTypeSimulatedUsers) \
    -objectiveValue         20 \
    -rampUpValue            5 \
    -sustainTime            20 \
    -rampDownTime           20
]

set serverTrafficNetworkMapping [::IxLoad new ixServerTrafficNetworkMapping \
    -network                $serverNetwork \
    -traffic                $serverTraffic \
    -matchClientTotalTime   1
]


#-----------------------------------------------------------------------
# Create the test and bind in the network-traffic mapping it is going
# to employ.
#-----------------------------------------------------------------------
set test [::IxLoad new ixTest \
    -name           		"Test" \
    -statsRequired  		1 \
    -enableResetPorts 		1 \
    -enableForceOwnership 	0 \
]

$test clientCommunityList.appendItem -object $clientTrafficNetworkMapping

$test serverCommunityList.appendItem -object $serverTrafficNetworkMapping


#-----------------------------------------------------------------------
# Create a test controller bound to the previosuly allocated
# chassis chain. This will eventually run the test we created earlier.
#-----------------------------------------------------------------------
set testController [::IxLoad new ixTestController -outputDir 1]

$testController setResultDir "RESULTS/$logName"

$testController run $test


#-----------------------------------------------------------------------
# Have the script (v)wait until the test is over.
#-----------------------------------------------------------------------
vwait ::ixTestControllerMonitor
puts $::ixTestControllerMonitor


#-----------------------------------------------------------------------
# Cleanup.
#-----------------------------------------------------------------------
$testController releaseConfigWaitFinish

::IxLoad delete $chassisChain

::IxLoad delete $clientNetwork

::IxLoad delete $serverNetwork

::IxLoad delete $clientTraffic

::IxLoad delete $serverTraffic

::IxLoad delete $Filter_1

::IxLoad delete $Filter_2

::IxLoad delete $GratARP_1

::IxLoad delete $GratARP_2

::IxLoad delete $TCP_1

::IxLoad delete $TCP_2

::IxLoad delete $DNS_1

::IxLoad delete $DNS_2

::IxLoad delete $Settings_1

::IxLoad delete $Settings_2

::IxLoad delete $Ethernet_1

::IxLoad delete $Ethernet_2

::IxLoad delete $my_ixNetEthernetELMPlugin_1

::IxLoad delete $my_ixNetEthernetELMPlugin_2

::IxLoad delete $MAC_VLAN_1

::IxLoad delete $MAC_VLAN_2

::IxLoad delete $IP_1

::IxLoad delete $IP_2

::IxLoad delete $IP_R1

::IxLoad delete $IP_R2

::IxLoad delete $MAC_R1

::IxLoad delete $MAC_R2

::IxLoad delete $VLAN_R1

::IxLoad delete $VLAN_R2

::IxLoad delete $clientTrafficNetworkMapping

::IxLoad delete $serverTrafficNetworkMapping

::IxLoad delete $test

::IxLoad delete $testController

::IxLoad delete $logger

::IxLoad delete $logEngine


#-----------------------------------------------------------------------
# Disconnect / Release application lock.
#-----------------------------------------------------------------------

}] {
    puts $errorInfo
}

::IxLoad disconnect
