#-----------------------------------------------------------------------
# Setup path and load IxLoad package.
#-----------------------------------------------------------------------
source ../setup_simple.tcl


#-----------------------------------------------------------------------
# Initialize IxLoad
# IxLoad connect should always be called, even for local scripts.
#-----------------------------------------------------------------------
::IxLoad connect $::IxLoadPrivate::SimpleSettings::remoteServer

# Once we've connected, make sure we disconnect, even if there's a problem
if [catch {


#-----------------------------------------------------------------------
# Loads plugins for specific protocols configured in this test.
#-----------------------------------------------------------------------
global ixAppPluginManager
$ixAppPluginManager load "HTTP"


#-----------------------------------------------------------------------
# Logger setup.
#-----------------------------------------------------------------------
set logtag "IxLoad-api"
set logName "HTTP_SSL_SM"
set logger [::IxLoad new ixLogger $logtag 1]
set logEngine [$logger getEngine]
$logEngine setLevels $::ixLogger(kLevelDebug) $::ixLogger(kLevelInfo)
$logEngine setFile $logName 2 256 1


#-----------------------------------------------------------------------
# Package require the stat collection utilities.
#-----------------------------------------------------------------------
package require statCollectorUtils
set scu_version [package require statCollectorUtils]
puts "statCollectorUtils package version = $scu_version"


#-----------------------------------------------------------------------
# Build Chassis Chain.
#-----------------------------------------------------------------------
set chassisChain [::IxLoad new ixChassisChain]

$chassisChain addChassis $::IxLoadPrivate::SimpleSettings::chassisName


#-----------------------------------------------------------------------
# Build Client Network.
#-----------------------------------------------------------------------
set clientNetwork [::IxLoad new ixNetworkGroup $chassisChain]

$clientNetwork config \
    -name                   "ClientNetwork" 

$clientNetwork globalPlugins.clear

set Filter_1 [::IxLoad new ixNetFilterPlugin]

$clientNetwork globalPlugins.appendItem -object $Filter_1

$Filter_1 config \
    -name                   "Filter_1" 

set GratARP_1 [::IxLoad new ixNetGratArpPlugin]

$clientNetwork globalPlugins.appendItem -object $GratARP_1

$GratARP_1 config \
    -name                   "GratARP_1" 

set TCP_1 [::IxLoad new ixNetTCPPlugin]

$clientNetwork globalPlugins.appendItem -object $TCP_1

$TCP_1 config \
    -tcp_tw_recycle         true \
    -tcp_keepalive_time     75 \
    -tcp_keepalive_intvl    7200 \
    -tcp_wmem_default       4096 \
    -tcp_port_min           1024 \
    -tcp_port_max           65535 \
    -tcp_window_scaling     false \
    -name                   "TCP_1" \
    -tcp_rmem_default       4096 

set DNS_1 [::IxLoad new ixNetDnsPlugin]

$clientNetwork globalPlugins.appendItem -object $DNS_1

$DNS_1 config \
    -name                   "DNS_1" 

$DNS_1 hostList.clear

$DNS_1 searchList.clear

$DNS_1 nameServerList.clear

set Settings_1 [::IxLoad new ixNetIxLoadSettingsPlugin]

$clientNetwork globalPlugins.appendItem -object $Settings_1

$Settings_1 config \
    -name                   "Settings_1" 

set Ethernet_1 [$clientNetwork getL1Plugin]

set my_ixNetEthernetELMPlugin_1 [::IxLoad new ixNetEthernetELMPlugin]

$Ethernet_1 config \
    -name                   "Ethernet /PHY_1" \
    -cardElm                $my_ixNetEthernetELMPlugin_1 

$Ethernet_1 childrenList.clear

set MAC_VLAN_1 [::IxLoad new ixNetL2EthernetPlugin]

$Ethernet_1 childrenList.appendItem -object $MAC_VLAN_1

$MAC_VLAN_1 config \
    -name                   "MAC/VLAN_1" 

$MAC_VLAN_1 childrenList.clear

set IP_1 [::IxLoad new ixNetIpV4V6Plugin]

$MAC_VLAN_1 childrenList.appendItem -object $IP_1

$IP_1 config \
    -name                   "IP_1" 

$IP_1 childrenList.clear

$IP_1 extensionList.clear

$MAC_VLAN_1 extensionList.clear

$Ethernet_1 extensionList.clear


#-----------------------------------------------------------------------
# Setting the ranges starting with the plugin on top of the stack.
#-----------------------------------------------------------------------
$IP_1 rangeList.clear

set IP_R1 [::IxLoad new ixNetIpV4V6Range]

$IP_1 rangeList.appendItem -object $IP_R1

$IP_R1 config \
    -count                  100 \
    -name                   "IP_R1" \
    -gatewayAddress         "0.0.0.0" \
    -autoMacGeneration      true \
    -prefix                 16 \
    -mss                    100 \
    -ipAddress              "198.18.2.1" 

set MAC_R1 [$IP_R1 getLowerRelatedRange "MacRange"]

$MAC_R1 config \
    -count                  100 \
    -mac                    "00:C6:12:02:01:00" \
    -name                   "MAC_R1" 

set VLAN_R1 [$IP_R1 getLowerRelatedRange "VlanIdRange"]

$VLAN_R1 config \
    -name                   "VLAN_R1" \
    -idIncrMode             1 \
    -priority               0 \
    -innerPriority          0 


#-----------------------------------------------------------------------
# Assign Ports to Client Network.
#-----------------------------------------------------------------------
$clientNetwork portList.appendItem \
    -chassisId              1 \
    -cardId                 $::IxLoadPrivate::SimpleSettings::clientPort(CARD_ID)\
    -portId                 $::IxLoadPrivate::SimpleSettings::clientPort(PORT_ID)


set serverNetwork [::IxLoad new ixNetworkGroup $chassisChain]
$serverNetwork config \
    -name                   "ServerNetwork" 

$serverNetwork globalPlugins.clear

set Filter_2 [::IxLoad new ixNetFilterPlugin]

$serverNetwork globalPlugins.appendItem -object $Filter_2

$Filter_2 config \
    -name                   "Filter_2" 

set GratARP_2 [::IxLoad new ixNetGratArpPlugin]

$serverNetwork globalPlugins.appendItem -object $GratARP_2

$GratARP_2 config \
    -name                   "GratARP_2" 

set TCP_2 [::IxLoad new ixNetTCPPlugin]

$serverNetwork globalPlugins.appendItem -object $TCP_2

$TCP_2 config \
    -tcp_tw_recycle         true \
    -tcp_keepalive_time     75 \
    -tcp_keepalive_intvl    7200 \
    -tcp_wmem_default       4096 \
    -tcp_port_min           1024 \
    -tcp_port_max           65535 \
    -tcp_window_scaling     false \
    -name                   "TCP_2" \
    -tcp_rmem_default       4096 

set DNS_2 [::IxLoad new ixNetDnsPlugin]

$serverNetwork globalPlugins.appendItem -object $DNS_2

$DNS_2 config \
    -name                   "DNS_2" 

$DNS_2 hostList.clear

$DNS_2 searchList.clear

$DNS_2 nameServerList.clear

set Settings_2 [::IxLoad new ixNetIxLoadSettingsPlugin]

$serverNetwork globalPlugins.appendItem -object $Settings_2

$Settings_2 config \
    -name                   "Settings_2" 

set Ethernet_2 [$serverNetwork getL1Plugin]

set my_ixNetEthernetELMPlugin_2 [::IxLoad new ixNetEthernetELMPlugin]


$Ethernet_2 config \
    -name                   "Ethernet /PHY_2" \
    -cardElm                $my_ixNetEthernetELMPlugin_2 

$Ethernet_2 childrenList.clear

set MAC_VLAN_2 [::IxLoad new ixNetL2EthernetPlugin]

$Ethernet_2 childrenList.appendItem -object $MAC_VLAN_2

$MAC_VLAN_2 config \
    -name                   "MAC/VLAN_2" 

$MAC_VLAN_2 childrenList.clear

set IP_2 [::IxLoad new ixNetIpV4V6Plugin]

$MAC_VLAN_2 childrenList.appendItem -object $IP_2

$IP_2 config \
    -name                   "IP_2" 

$IP_2 childrenList.clear

$IP_2 extensionList.clear

$MAC_VLAN_2 extensionList.clear

$Ethernet_2 extensionList.clear


#-----------------------------------------------------------------------
# Setting the ranges starting with the plugin on top of the stack.
#-----------------------------------------------------------------------
$IP_2 rangeList.clear

set IP_R2 [::IxLoad new ixNetIpV4V6Range]

$IP_2 rangeList.appendItem -object $IP_R2

$IP_R2 config \
    -name                   "IP_R2" \
    -gatewayAddress         "0.0.0.0" \
    -prefix                 16 \
    -mss                    100 \
    -ipAddress              "198.18.200.1" 

set MAC_R2 [$IP_R2 getLowerRelatedRange "MacRange"]

$MAC_R2 config \
    -mac                    "00:C6:12:02:02:00" \
    -name                   "MAC_R2" 

set VLAN_R2 [$IP_R2 getLowerRelatedRange "VlanIdRange"]

$VLAN_R2 config \
    -name                   "VLAN_R2" \
    -idIncrMode             1 \
    -priority               0 \
    -innerPriority          0 


$serverNetwork portList.appendItem \
    -chassisId  1 \
    -cardId     $::IxLoadPrivate::SimpleSettings::serverPort(CARD_ID)\
    -portId     $::IxLoadPrivate::SimpleSettings::serverPort(PORT_ID)


#-----------------------------------------------------------------------
# SSL Key & Certificate.
#-----------------------------------------------------------------------
set Secured_RSA_privateKeyPassword "ixload"

set Secured_RSA_cert_512 "-----BEGIN CERTIFICATE-----
MIIDoTCCAokCAQIwDQYJKoZIhvcNAQEFBQAwgZcxCzAJBgNVBAYTAlVTMQswCQYD
VQQIDAJDQTEMMAoGA1UEBwwDQ0FMMQ0wCwYDVQQKDARJWElBMQwwCgYDVQQLDANC
UFMxKDAmBgNVBAMMH1Jvb3RfQ0FfVW5zZWN1cmVkX1JTQV9jZXJ0XzIwNDgxJjAk
BgkqhkiG9w0BCQEWF3Jvb3QtY2EtcnNhQGl4aWFjb20uY29tMCAXDTE3MDUzMDA4
MDE1MloYDzIxMTcwNTA2MDgwMTUyWjCBkjELMAkGA1UEBhMCSU4xCzAJBgNVBAgM
AldCMQwwCgYDVQQHDANLT0wxDTALBgNVBAoMBElYSUExDzANBgNVBAsMBklYTE9B
RDEoMCYGA1UEAwwfQ0FfU2lnbmVkX1NlY3VyZWRfUlNBX2NlcnRfMjA0ODEeMBwG
CSqGSIb3DQEJARYPYWJjQGl4aWFjb20uY29tMIIBIjANBgkqhkiG9w0BAQEFAAOC
AQ8AMIIBCgKCAQEA0Vuiulf8tpnWWnigv8GdLpMa5v0Z0Pfm1Ggm+axJnqv1FxrD
clIAti7Vrthbz07C5Kgz4Gi8EUudu3iNLLmJ9Eq0ZMruXlqrB+gr+zBANJ6znchr
Y2/AYkSe/vZLt3wiTHwPNjJH0X4yjowh8qhvHDVczt1ga0XhZScU49wVOzTvfxV+
g2GsswwxvwEt7BqPhiCKprgAWZovC6Ml87UzlvusulwqmxWCQwR41gm2CX2XxRKu
I4njexzgrvPGElN0WtZ8U8vJ4/RSg+phzqEUFZzQdKgcc8DorM01cM/Z08BuCBZU
1s23YCHF+Koj8ADWTCI6DKrxjNU8Fk17reMgYQIDAQABMA0GCSqGSIb3DQEBBQUA
A4IBAQDYtAPUaWQrX2AGsmKGt8XOFdwOVz8NaHcEQju9/LxwRepXZWHSN9lc1DHG
6Ue9kyz1pP6FHLNM2+RGE/2DMmxMcaeWb+AhFfClXH5wnyBA+cxu7NZDbPC0ZeRH
OBpauWaZjZhkZAsWysqpSCNQuBpZVcT6OKzfKii8z1I+Bi8QIr+rAt9N4oS/fBMC
Z8+rGhqUffGMsw3xBy1W8QyVN6Utf6oEmXAOKvjQ0Gdx/gKc6BCF6LaswGMjaVsv
sxVyLAWccnuK5Ggo58F3Eqk6m8M8gpk+W2rjOPKnY5f+dTLlcPxRXPwmOTGnF/Iz
YIEoR7AfbAEQelKyGZWY68HMxnNe
-----END CERTIFICATE-----"

set Secured_RSA_key_512 "-----BEGIN RSA PRIVATE KEY-----
Proc-Type: 4,ENCRYPTED
DEK-Info: DES-EDE3-CBC,A3E5A86CF34464DA

ArZ/fghkImKAUu/5HdbWKj9jSQkBz+YyYFcXe0z1CWZrSgkUheNpHoeQDoXfo0Pl
PHQbzDs86MsiiusSLD+Cu0iyo761JhffKXS57dAcWuZauN+sypTEBM9U6byFMQl4
5a8WC1F9mQNqZoWYG7VhGoJKupdZMjiDRaIMHQB0uGc7ycxiPZ3UoQuYgdjE/JTL
9thXOJOPbnDB9meu7bNX2nAGS9lN/tUHeVBEWJnokPUSmvJlhlhDdwJ4vAN6kNzr
3oWvu+/zN8MIZte6EmJcPTDb8xPM+CldoszzI/kx7+cKBvX6ugsEC2qowrdBxsrj
TMGN/3dMbeNDZlCLOGtDEuRq2pV6QUwzHCay1BN07CX03pMILUBNtZScaJEPv7wu
aPT0V5V+R8YsSKn691YGNO1huDKMjCNRcL5+OwQL5nTxjnBtAq8Ce+6AGnx4EZ9H
aZHQdmLyW7zLn4d/B8Z4pf+GW2UeC6I1/QlGPRo6hm7G+YTy6I2fPfzQn921jPSq
5vk1LFomwz5wbeqs2nZVDfIja5Ui70pxnItsSgPuBHbmj0zPjfNT9eLXG0B3LNSE
0KBaxyHSJw/y4k+34eNXTWKd9E4V33mdqI0shjdWei2W44apsRlSWK5lm5MzUq1f
WbwP5A6t+O/OWRyUuy/W6N/KFHEQ+Wu8kvVlbLkSYg1D0ND7TXLT5dgxZJH+0AoU
7IiMwfg0gVQP83smArnRp51GttIxnxnQCUSNDHkEAI26y6wopRwN1InW0eaSdlir
a+sA6jhB5kDQbmwx2prcr5NF9ij6nevULu4p7QUKzoggCHb7h5sGOgoZrmN1MAxJ
jyx+6674zM0kk6BJFsJr7at4UxufZDW7bQIDRU5ajKAYtxz5dnORo3T0+8ULpmiP
/f5lebAMAs6fBPDmzEUNkuZpjJXhKFoX6hj9EnLuE+aKEEbcZCuP5viekq9tJGaC
9Onr8cpCcmrjHshDXTadrwzDAmZScmhYZ7b4URIuXFz7Oi5yv0sNi0sXnPTgGCx4
MvcFXECLi3Y4FoG+iC70A2Tuqi5HjXydaUBv5VYkVLiaofN6xBXk+iOswb0eETx9
6q7hpV3E8k6BHPUP42hZ+E4yz5UB72VZl3vC6m9Ftd8yarCw+pK13dCLRSBB6OJq
zpEpdtiKWZUTQEw1ZOw5nb3Htv93cUpgo0skkTW4Q1NBuaPoljB1pV2DFd4ux9gY
EI9kXutxkk4X5EXrxC3OwSM1Bpt3WyYB9H/vzMY+1DwuIEHsFc6mErOfHOZ4M7sI
LtxL2b3pScViytUVNKGkQowWkpwPPxKToTG11hT2qLL4GxkD+mrSPnxDnfFYP98G
m1XIaU1QhAL61lHTApbRopU3vo2Opd807GjZD52Laa+HyezQQxj5htg3lvEpWmEM
dploaB2uRndKu3MPwfl0QcKbHroYmnqbqrY4XWA7l2iV7z2cI5chZ/hjzfHkZqid
nPgq+NAqqqwhdfRvmNPYX8sFO94kOmfAnqU968Kj0MUSu40W05wMuMLd5wLRtlyg
mF2RF9nGlBkaOm3815j61iNACyqqEh+uV+8mcfSliLUfV+uHnGvUGCbEQlUVTe5U
-----END RSA PRIVATE KEY-----"
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# HTTP Client Configuration.
#-----------------------------------------------------------------------
set clientTraffic [::IxLoad new ixClientTraffic -name "ClientTraffic"]

$clientTraffic agentList.appendItem \
    -name                   "HTTPClient" \
    -protocol               "HTTP" \
    -type                   "Client" \
    -maxSessions            3 \
    -httpVersion            $::HTTP_Client(kHttpVersion10) \
    -keepAlive              0 \
    -maxPersistentRequests  3 \
    -followHttpRedirects    0 \
    -enableCookieSupport    0 \
    -enableHttpProxy        0 \
    -enableHttpsProxy       0 \
    -browserEmulation       $::HTTP_Client(kBrowserTypeIE5) \
    -enableSsl              1 \
    -sslVersion             $::HTTP_Client(kSslVersion3) \
    -sequentialSessionReuse 2 \
    -clientCiphers          "DHE-RSA-AES128-SHA" \
    -privateKeyPassword     $Secured_RSA_privateKeyPassword \
    -privateKey             $Secured_RSA_key_512 \
    -certificate            $Secured_RSA_cert_512

foreach {pageObject destination} {
    "/4k.htm" "ServerTraffic_HTTPServer"
    "/8k.htm" "ServerTraffic_HTTPServer"
    "/128k.htm" "ServerTraffic_HTTPServer"
} {
    $clientTraffic agentList(0).actionList.appendItem  \
        -command        "GET(SSL)" \
        -destination    $destination \
        -pageObject     $pageObject
}


#-----------------------------------------------------------------------
# HTTP Server Configuration.
#-----------------------------------------------------------------------
set serverTraffic [::IxLoad new ixServerTraffic -name "ServerTraffic"]

$serverTraffic agentList.appendItem \
    -name                   "HTTPServer" \
    -protocol               "HTTP" \
    -type                   "Server" \
    -acceptSslConnections   1 \
    -httpsPort              443 \
    -ServerCiphers          "DHE-RSA-AES128-SHA" \
    -enableDHsupport        1 \
    -privateKeyPassword     $Secured_RSA_privateKeyPassword \
    -privateKey             $Secured_RSA_key_512 \
    -certificate            $Secured_RSA_cert_512

for {set idx 0} {$idx < [$serverTraffic agentList(0).responseHeaderList.indexCount]} {incr idx} {
    set response [$serverTraffic agentList(0).responseHeaderList.getItem $idx]
    if {[$response cget -name] == "200_OK"} {
        set response200ok $response
    }
    if {[$response cget -name] == "404_PageNotFound"} {
        set response404_PageNotFound $response
    }
}

$serverTraffic agentList(0).webPageList.clear

$serverTraffic agentList(0).webPageList.appendItem \
    -page                   "/4k.html" \
    -payloadType            "range" \
    -payloadSize            "4096-4096" \
    -response               $response200ok

$serverTraffic agentList(0).webPageList.appendItem \
    -page                   "/8k.html" \
    -payloadType            "range" \
    -payloadSize            "8192-8192" \
    -response               $response404_PageNotFound

$serverTraffic agentList(0).webPageList.appendItem \
    -page                   "/128k.html" \
    -payloadType            "range" \
    -payloadSize            "131072" \
    -response               $response200ok


#-----------------------------------------------------------------------
# Create a client and server mapping and bind into the
# network and traffic that they will be employing.
#-----------------------------------------------------------------------
set clientTrafficNetworkMapping [::IxLoad new ixClientTrafficNetworkMapping \
    -network                $clientNetwork \
    -traffic                $clientTraffic \
    -objectiveType          $::ixObjective(kObjectiveTypeSimulatedUsers) \
    -objectiveValue         20 \
    -rampUpValue            5 \
    -sustainTime            20 \
    -rampDownTime           20
]

set serverTrafficNetworkMapping [::IxLoad new ixServerTrafficNetworkMapping \
    -network                $serverNetwork \
    -traffic                $serverTraffic \
    -matchClientTotalTime   1
]


#-----------------------------------------------------------------------
# Create the test and bind in the network-traffic mapping it is going
# to employ.
#-----------------------------------------------------------------------
set test [::IxLoad new ixTest \
    -name                   "Test" \
    -statsRequired          1 \
    -enableResetPorts       1 \
    -enableForceOwnership   1 \
]

$test clientCommunityList.appendItem -object $clientTrafficNetworkMapping
$test serverCommunityList.appendItem -object $serverTrafficNetworkMapping


#-----------------------------------------------------------------------
# Create a test controller bound to the previosuly allocated
# chassis chain. This will eventually run the test we created earlier.
#-----------------------------------------------------------------------
set testController [::IxLoad new ixTestController -outputDir 1]

$testController setResultDir "[pwd]/RESULTS/$logName"


#-----------------------------------------------------------------------
# Set up stat collection.
#-----------------------------------------------------------------------
set NS statCollectorUtils

set ::test_server_handle [$testController getTestServerHandle]

${NS}::Initialize -testServerHandle $::test_server_handle


#-----------------------------------------------------------------------
# Clear any stats that may have been registered previously.
#-----------------------------------------------------------------------
${NS}::ClearStats


#-----------------------------------------------------------------------
# Define the stats we would like to collect.
#-----------------------------------------------------------------------
${NS}::AddStat \
    -caption "Watch_Stat_1" \
    -statSourceType "HTTP Client" \
    -statName "HTTP Bytes Sent" \
    -aggregationType kSum \
    -filterList {}

${NS}::AddStat \
    -caption "Watch_Stat_2" \
    -statSourceType "HTTP Client" \
    -statName "HTTP Bytes Received" \
    -aggregationType kSum \
    -filterList {}

${NS}::AddStat \
    -caption "Watch_Stat_3" \
    -statSourceType "HTTP Client" \
    -statName "HTTP Time To Last Byte (us)" \
    -aggregationType kWeightedAverage \
    -filterList {}

${NS}::AddStat \
    -caption "Watch_Stat_4" \
    -statSourceType "HTTP Client" \
    -statName "HTTP Bytes Sent" \
    -aggregationType kRate \
    -filterList {}

${NS}::AddStat \
    -caption "Watch_Stat_5" \
    -statSourceType "HTTP Client" \
    -statName "HTTP Bytes Received" \
    -aggregationType kRate \
    -filterList {}


#-----------------------------------------------------------------------
# Start the collector (runs in the tcl event loop).
#-----------------------------------------------------------------------
proc ::my_stat_collector_command {args} {
    puts "====================================="
    puts "INCOMING STAT RECORD >>> $args"
    puts "Len = [llength $args]"
    puts  [lindex $args 0]
    puts  [lindex $args 1]
    puts "====================================="
}
${NS}::StartCollector -command ::my_stat_collector_command

$testController run $test


#-----------------------------------------------------------------------
# Have the script (v)wait until the test is over.
#-----------------------------------------------------------------------
vwait ::ixTestControllerMonitor
puts $::ixTestControllerMonitor


#-----------------------------------------------------------------------
# Stop the collector (running in the tcl event loop).
#-----------------------------------------------------------------------
${NS}::StopCollector


#-----------------------------------------------------------------------
# Cleanup.
#-----------------------------------------------------------------------
$testController generateReport -detailedReport 1 -format "PDF;HTML"

$testController collectDebugLogs

$testController releaseConfigWaitFinish

::IxLoad delete $chassisChain

::IxLoad delete $clientNetwork

::IxLoad delete $serverNetwork

::IxLoad delete $clientTraffic

::IxLoad delete $serverTraffic

::IxLoad delete $Filter_1

::IxLoad delete $Filter_2

::IxLoad delete $GratARP_1

::IxLoad delete $GratARP_2

::IxLoad delete $TCP_1

::IxLoad delete $TCP_2

::IxLoad delete $DNS_1

::IxLoad delete $DNS_2

::IxLoad delete $Settings_1

::IxLoad delete $Settings_2

::IxLoad delete $Ethernet_1

::IxLoad delete $Ethernet_2

::IxLoad delete $my_ixNetEthernetELMPlugin_1

::IxLoad delete $my_ixNetEthernetELMPlugin_2

::IxLoad delete $MAC_VLAN_1

::IxLoad delete $MAC_VLAN_2

::IxLoad delete $IP_1

::IxLoad delete $IP_2

::IxLoad delete $IP_R1

::IxLoad delete $IP_R2

::IxLoad delete $MAC_R1

::IxLoad delete $MAC_R2

::IxLoad delete $VLAN_R1

::IxLoad delete $VLAN_R2

::IxLoad delete $clientTrafficNetworkMapping

::IxLoad delete $serverTrafficNetworkMapping

::IxLoad delete $test

::IxLoad delete $testController

::IxLoad delete $logger

::IxLoad delete $logEngine


#-----------------------------------------------------------------------
# Disconnect / Release application lock.
#-----------------------------------------------------------------------

}] {
    puts $errorInfo
}

::IxLoad disconnect
