#-----------------------------------------------------------------------
# Setup path and load IxLoad package.
#-----------------------------------------------------------------------
source ../setup_simple.tcl


#-----------------------------------------------------------------------
# Initialize IxLoad
# IxLoad connect should always be called, even for local scripts.
#-------------------------------------------------------------------
::IxLoad connect $::IxLoadPrivate::SimpleSettings::remoteServer

# Once we've connected, make sure we disconnect, even if there's a problem
if [catch {


#-----------------------------------------------------------------------
# Loads plugins for specific protocols configured in this test.
#-----------------------------------------------------------------------
global ixAppPluginManager
$ixAppPluginManager load "LDAP"


#-----------------------------------------------------------------------
# Logger setup.
#-----------------------------------------------------------------------
set logtag "IxLoad-api"
set logName "LDAP_SM"
set logger [::IxLoad new ixLogger $logtag 1]
set logEngine [$logger getEngine]
$logEngine setLevels $::ixLogger(kLevelDebug) $::ixLogger(kLevelInfo)
$logEngine setFile $logName 2 256 1


#-----------------------------------------------------------------------
# Build Chassis Chain.
#-----------------------------------------------------------------------
set chassisChain [::IxLoad new ixChassisChain]

$chassisChain addChassis $::IxLoadPrivate::SimpleSettings::chassisName


#-----------------------------------------------------------------------
# Build Client Network.
#-----------------------------------------------------------------------
set clientNetwork [::IxLoad new ixNetworkGroup $chassisChain]

$clientNetwork config \
	-name                 	"ClientNetwork"

$clientNetwork globalPlugins.clear

set Filter_1 [::IxLoad new ixNetFilterPlugin]

$clientNetwork globalPlugins.appendItem -object $Filter_1

$Filter_1 config \
	-name                   "Filter_1"

set GratARP_1 [::IxLoad new ixNetGratArpPlugin]

$clientNetwork globalPlugins.appendItem -object $GratARP_1

$GratARP_1 config \
	-name					"GratARP_1"

set TCP_1 [::IxLoad new ixNetTCPPlugin]

$clientNetwork globalPlugins.appendItem -object $TCP_1

$TCP_1 config \
	-tcp_tw_recycle			true \
	-tcp_keepalive_time		75 \
	-tcp_keepalive_intvl	7200 \
	-tcp_wmem_default		4096 \
	-tcp_port_min			1024 \
	-tcp_port_max			65535 \
	-tcp_window_scaling		false \
	-name					"TCP_1" \
	-tcp_rmem_default		4096

set DNS_1 [::IxLoad new ixNetDnsPlugin]

$clientNetwork globalPlugins.appendItem -object $DNS_1

$DNS_1 config \
	-name                   "DNS_1"

$DNS_1 hostList.clear

$DNS_1 searchList.clear

$DNS_1 nameServerList.clear

set Settings_1 [::IxLoad new ixNetIxLoadSettingsPlugin]

$clientNetwork globalPlugins.appendItem -object $Settings_1

$Settings_1 config \
	-name					"Settings_1"

set Ethernet_1 [$clientNetwork getL1Plugin]

set my_ixNetEthernetELMPlugin_1 [::IxLoad new ixNetEthernetELMPlugin]

$Ethernet_1 config \
	-name					"Ethernet /PHY_1" \
	-cardElm				$my_ixNetEthernetELMPlugin_1

$Ethernet_1 childrenList.clear

set MAC_VLAN_1 [::IxLoad new ixNetL2EthernetPlugin]

$Ethernet_1 childrenList.appendItem -object $MAC_VLAN_1

$MAC_VLAN_1 config \
	-name                   "MAC/VLAN_1"

$MAC_VLAN_1 childrenList.clear

set IP_1 [::IxLoad new ixNetIpV4V6Plugin]

$MAC_VLAN_1 childrenList.appendItem -object $IP_1

$IP_1 config \
	-name					"IP_1"

$IP_1 childrenList.clear

$IP_1 extensionList.clear

$MAC_VLAN_1 extensionList.clear

$Ethernet_1 extensionList.clear


#-----------------------------------------------------------------------
# Setting the ranges starting with the plugin on top of the stack.
#-----------------------------------------------------------------------
$IP_1 rangeList.clear

set IP_R1 [::IxLoad new ixNetIpV4V6Range]

$IP_1 rangeList.appendItem -object $IP_R1

$IP_R1 config \
	-count                  10 \
	-name                   "IP_R1" \
	-gatewayAddress         "0.0.0.0" \
	-prefix                 16 \
	-mss                    100 \
	-ipAddress              "198.18.0.1"

set MAC_R1 [$IP_R1 getLowerRelatedRange "MacRange"]

$MAC_R1 config \
	-count                  10 \
	-mac                    "00:C6:12:02:01:00" \
	-name                   "MAC_R1"

set VLAN_R1 [$IP_R1 getLowerRelatedRange "VlanIdRange"]

$VLAN_R1 config \
	-name                   "VLAN_R1" \
	-idIncrMode             1 \
	-priority               0 \
	-innerPriority          0


#-----------------------------------------------------------------------
# Assign Ports to Client Network.
#-----------------------------------------------------------------------
$clientNetwork portList.appendItem \
    -chassisId  			1 \
    -cardId     			$::IxLoadPrivate::SimpleSettings::clientPort(CARD_ID)\
    -portId     			$::IxLoadPrivate::SimpleSettings::clientPort(PORT_ID)


#-----------------------------------------------------------------------
# LDAP Client Configuration.
#-----------------------------------------------------------------------
set clientTraffic [::IxLoad new ixClientTraffic -name "ClientTraffic"]

$clientTraffic agentList.appendItem \
    -name               	"LDAPClient" \
    -protocol           	"ldap" \
    -type               	"Client"

$clientTraffic agentList(0).pm.cmdList.appendItem \
    -id                 	"BIND"  \
    -serverAddr        		"198.18.254.253" \
    -name              		"cn=Manager,dc=ixiacom,dc=com" \
    -authType          		"CLEARTEXT Password" \
    -password          		"secret"


#-----------------------------------------------------------------------
# Create a client and server mapping and bind into the
# network and traffic that they will be employing.
#-----------------------------------------------------------------------
set clientTrafficNetworkMapping [::IxLoad new ixClientTrafficNetworkMapping \
    -network                $clientNetwork \
    -traffic                $clientTraffic \
    -objectiveType          $::ixObjective(kObjectiveTypeSimulatedUsers) \
    -objectiveValue         20 \
    -rampUpValue            5 \
    -sustainTime            60 \
    -rampDownTime           20
]


#-----------------------------------------------------------------------
# Create the test and bind in the network-traffic mapping it is going
# to employ.
#-----------------------------------------------------------------------
set test [::IxLoad new ixTest \
    -name           		"Test" \
    -statsRequired  		1 \
    -enableResetPorts 		1 \
    -enableForceOwnership 	0 \
]

$test clientCommunityList.appendItem -object $clientTrafficNetworkMapping


#-----------------------------------------------------------------------
# Create a test controller bound to the previosuly allocated
# chassis chain. This will eventually run the test we created earlier.
#-----------------------------------------------------------------------
set testController [::IxLoad new ixTestController -outputDir 1]

$testController setResultDir "RESULTS/$logName"

$testController run $test


#-----------------------------------------------------------------------
# Have the script (v)wait until the test is over.
#-----------------------------------------------------------------------
vwait ::ixTestControllerMonitor
puts $::ixTestControllerMonitor


#-----------------------------------------------------------------------
# Cleanup.
#-----------------------------------------------------------------------
$testController releaseConfigWaitFinish

::IxLoad delete $chassisChain

::IxLoad delete $clientNetwork

::IxLoad delete $clientTraffic

::IxLoad delete $Filter_1

::IxLoad delete $GratARP_1

::IxLoad delete $TCP_1

::IxLoad delete $DNS_1

::IxLoad delete $Settings_1

::IxLoad delete $Ethernet_1

::IxLoad delete $my_ixNetEthernetELMPlugin_1

::IxLoad delete $MAC_VLAN_1

::IxLoad delete $IP_1

::IxLoad delete $IP_R1

::IxLoad delete $MAC_R1

::IxLoad delete $VLAN_R1

::IxLoad delete $clientTrafficNetworkMapping

::IxLoad delete $test

::IxLoad delete $testController

::IxLoad delete $logger

::IxLoad delete $logEngine


#-----------------------------------------------------------------------
# Disconnect / Release application lock.
#-----------------------------------------------------------------------

}] {
    puts $errorInfo
}

::IxLoad disconnect
