#-----------------------------------------------------------------------
# Setup path and load IxLoad package.
#-----------------------------------------------------------------------
source ../setup_simple.tcl


#-----------------------------------------------------------------------
# Initialize IxLoad
# IxLoad connect should always be called, even for local scripts.
#-----------------------------------------------------------------------
::IxLoad connect $::IxLoadPrivate::SimpleSettings::remoteServer


# Once we've connected, make sure we disconnect, even if there's a problem
if [catch {


#-----------------------------------------------------------------------
# Loads plugins for specific protocols configured in this test.
#-----------------------------------------------------------------------    
global ixAppPluginManager
$ixAppPluginManager load "SIP"


#-----------------------------------------------------------------------
# Logger setup.
#-----------------------------------------------------------------------
set logtag "IxLoad-api"
set logName  "SIP_SM"
set logger [::IxLoad new ixLogger $logtag 1]
set logEngine [$logger getEngine]
$logEngine setLevels $::ixLogger(kLevelDebug) $::ixLogger(kLevelInfo)
$logEngine setFile $logName 2 256 1


#-----------------------------------------------------------------------
# Build Chassis Chain.
#-----------------------------------------------------------------------
set chassisChain [::IxLoad new ixChassisChain]

$chassisChain addChassis $::IxLoadPrivate::SimpleSettings::chassisName


#-----------------------------------------------------------------------
# Build Client Network.
#-----------------------------------------------------------------------
set clientNetwork [::IxLoad new ixNetworkGroup $chassisChain]
$clientNetwork config \
    -name                   "ClientNetwork" 

$clientNetwork globalPlugins.clear

set Filter_1 [::IxLoad new ixNetFilterPlugin]

$clientNetwork globalPlugins.appendItem -object $Filter_1

$Filter_1 config \
    -name                   "Filter_1" 

set GratARP_1 [::IxLoad new ixNetGratArpPlugin]

$clientNetwork globalPlugins.appendItem -object $GratARP_1

$GratARP_1 config \
    -enabled                true \
    -name                   "GratARP_1" 

set TCP_1 [::IxLoad new ixNetTCPPlugin]

$clientNetwork globalPlugins.appendItem -object $TCP_1

$TCP_1 config \
    -tcp_tw_recycle         true \
    -tcp_keepalive_time     75 \
    -tcp_keepalive_intvl    7200 \
    -tcp_wmem_default       4096 \
    -tcp_port_min           1024 \
    -tcp_port_max           65535 \
    -tcp_window_scaling     false \
    -name                   "TCP_1" \
    -tcp_rmem_default       4096 

set DNS_1 [::IxLoad new ixNetDnsPlugin]

$clientNetwork globalPlugins.appendItem -object $DNS_1

$DNS_1 config \
    -name                   "DNS_1" 

$DNS_1 hostList.clear

$DNS_1 searchList.clear

$DNS_1 nameServerList.clear

set Settings_1 [::IxLoad new ixNetIxLoadSettingsPlugin]

$clientNetwork globalPlugins.appendItem -object $Settings_1

$Settings_1 config \
    -name                   "Settings_1" 

set Ethernet_1 [$clientNetwork getL1Plugin]

set my_ixNetEthernetELMPlugin_1 [::IxLoad new ixNetEthernetELMPlugin]

$Ethernet_1 config \
    -name                   "Ethernet /PHY_1" \
    -cardElm                $my_ixNetEthernetELMPlugin_1 

$Ethernet_1 childrenList.clear

set MAC_VLAN_1 [::IxLoad new ixNetL2EthernetPlugin]

$Ethernet_1 childrenList.appendItem -object $MAC_VLAN_1

$MAC_VLAN_1 config \
    -name                   "MAC/VLAN_1" 

$MAC_VLAN_1 childrenList.clear

set IP_1 [::IxLoad new ixNetIpV4V6Plugin]

$MAC_VLAN_1 childrenList.appendItem -object $IP_1

$IP_1 config \
    -name                   "IP_1" 

$IP_1 childrenList.clear

$IP_1 extensionList.clear

$MAC_VLAN_1 extensionList.clear

$Ethernet_1 extensionList.clear


#-----------------------------------------------------------------------
# Setting the ranges starting with the plugin on top of the stack.
#-----------------------------------------------------------------------
$IP_1 rangeList.clear

set IP_R1 [::IxLoad new ixNetIpV4V6Range]

$IP_1 rangeList.appendItem -object $IP_R1

$IP_R1 config \
    -name                   "IP_R1" \
    -gatewayAddress         "0.0.0.0" \
    -prefix                 16 \
    -mss                    100 \
    -ipAddress              "198.18.0.1" 

set MAC_R1 [$IP_R1 getLowerRelatedRange "MacRange"]

$MAC_R1 config \
    -mac                    "00:C6:12:02:01:00" \
    -name                   "MAC_R1" 

set VLAN_R1 [$IP_R1 getLowerRelatedRange "VlanIdRange"]

$VLAN_R1 config \
    -name                   "VLAN_R1" \
    -idIncrMode             1 \
    -priority               0 \
    -innerPriority          0 


#-----------------------------------------------------------------------
# Assign Ports to Client Network.
#-----------------------------------------------------------------------
$clientNetwork portList.appendItem \
    -chassisId              1 \
    -cardId                 $::IxLoadPrivate::SimpleSettings::clientPort(CARD_ID)\
    -portId                 $::IxLoadPrivate::SimpleSettings::clientPort(PORT_ID)


#-----------------------------------------------------------------------
# Build Server Network.
#-----------------------------------------------------------------------
set serverNetwork [::IxLoad new ixNetworkGroup $chassisChain]

$serverNetwork config \
    -name                   "ServerNetwork" 

$serverNetwork globalPlugins.clear

set Filter_2 [::IxLoad new ixNetFilterPlugin]

$serverNetwork globalPlugins.appendItem -object $Filter_2

$Filter_2 config \
    -name                   "Filter_2" 

set GratARP_2 [::IxLoad new ixNetGratArpPlugin]

$serverNetwork globalPlugins.appendItem -object $GratARP_2

$GratARP_2 config \
    -enabled                true \
    -name                   "GratARP_2" 

set TCP_2 [::IxLoad new ixNetTCPPlugin]

$serverNetwork globalPlugins.appendItem -object $TCP_2

$TCP_2 config \
    -tcp_tw_recycle         true \
    -tcp_keepalive_time     75 \
    -tcp_keepalive_intvl    7200 \
    -tcp_wmem_default       4096 \
    -tcp_port_min           1024 \
    -tcp_port_max           65535 \
    -tcp_window_scaling     false \
    -name                   "TCP_2" \
    -tcp_rmem_default       4096 

set DNS_2 [::IxLoad new ixNetDnsPlugin]

$serverNetwork globalPlugins.appendItem -object $DNS_2

$DNS_2 config \
    -name                   "DNS_2" 

$DNS_2 hostList.clear

$DNS_2 searchList.clear

$DNS_2 nameServerList.clear

set Settings_2 [::IxLoad new ixNetIxLoadSettingsPlugin]

$serverNetwork globalPlugins.appendItem -object $Settings_2

$Settings_2 config \
    -name                   "Settings_2" 

set Ethernet_2 [$serverNetwork getL1Plugin]

set my_ixNetEthernetELMPlugin_2 [::IxLoad new ixNetEthernetELMPlugin]

$Ethernet_2 config \
    -name                   "Ethernet /PHY_2" \
    -cardElm                $my_ixNetEthernetELMPlugin_2 

$Ethernet_2 childrenList.clear

set MAC_VLAN_2 [::IxLoad new ixNetL2EthernetPlugin]

$Ethernet_2 childrenList.appendItem -object $MAC_VLAN_2

$MAC_VLAN_2 config \
    -name                   "MAC/VLAN_2" 

$MAC_VLAN_2 childrenList.clear

set IP_2 [::IxLoad new ixNetIpV4V6Plugin]

$MAC_VLAN_2 childrenList.appendItem -object $IP_2

$IP_2 config \
    -name                   "IP_2" 

$IP_2 childrenList.clear

$IP_2 extensionList.clear

$MAC_VLAN_2 extensionList.clear

$Ethernet_2 extensionList.clear


#-----------------------------------------------------------------------
# Setting the ranges starting with the plugin on top of the stack.
#-----------------------------------------------------------------------
$IP_2 rangeList.clear

set IP_R2 [::IxLoad new ixNetIpV4V6Range]

$IP_2 rangeList.appendItem -object $IP_R2

$IP_R2 config \
    -name                   "IP_R2" \
    -gatewayAddress         "0.0.0.0" \
    -prefix                 16 \
    -ipAddress              "198.18.0.101" 

set MAC_R2 [$IP_R2 getLowerRelatedRange "MacRange"]

$MAC_R2 config \
    -mac                    "00:C6:12:02:02:00" \
    -name                   "MAC_R2" 

set VLAN_R2 [$IP_R2 getLowerRelatedRange "VlanIdRange"]

$VLAN_R2 config \
    -name                   "VLAN_R2" \
    -idIncrMode             1 \
    -priority               0 \
    -innerPriority          0 


#-----------------------------------------------------------------------
# Assign Ports to Server Network.
#-----------------------------------------------------------------------
$serverNetwork portList.appendItem \
    -chassisId              1 \
    -cardId                 $::IxLoadPrivate::SimpleSettings::serverPort(CARD_ID) \
    -portId                 $::IxLoadPrivate::SimpleSettings::serverPort(PORT_ID)


#-----------------------------------------------------------------------
# SIP Client Configuration.
#-----------------------------------------------------------------------
set clientTraffic [::IxLoad new ixClientTraffic -name "ClientTraffic"]

$clientTraffic agentList.appendItem \
    -name                   "SIPClient" \
    -protocol               "SIP" \
    -type                   "Client"


$clientTraffic agentList(0).pm.generalSettings.config \
     -szAuthUsername         "user" \
     -szAuthPassword         "password" \
     -szAuthDomain           "domain" \
     -szTransport            "UDP" \
     -nUdpPort               5060 \
     -nTcpPort               5060 \
     -nUdpMaxSize            1024 \
     -szRegistrar            "127.0.0.1:5060" \
     -bRegBefore             0

 $clientTraffic agentList(0).pm.contentOfMessages.config \
     -bRoute                 0 \
     -szRoute                "Route: &lt;sip:p1.example.com;lr&gt;,&lt;sip:p2.domain.com;lr&gt;" \
     -bCompact               0 \
     -bFolding               0 \
     -bScattered             0 \
     -bAdvisable             0 \
     -bOptional              0 \
     -bBestPerformance       1 \
     -szREQUESTURI           "sip:id@IP" \
     -szFROM                 "sip:id@IP" \
     -szTO                   "sip:id@IP" \
     -szCONTACT              "sip:id@IP"

$clientTraffic agentList(0).pm.stateMachine.config \
     -nTimersT1              500 \
     -nTimersT2              4000 \
     -nTimersT4              5000 \
     -nTimersTC              180000 \
     -nTimersTD              32000 

#-----------------------------------------------------------------------
# pm.mediaSettings.config
#
# Configuring codec and codec settings:
#-szCodecName - defines the codec to be used for RTP streams and can have the following values:
#   "G711ALaw" - for G.711 A-law
#   "G711ULaw" - for G.711 mu-law
#   "G729A"  - for G.729A
#   "G729B"  - for G.729B
#   "G726"   - for G.726
#   "G723_1" - for G.723.1
#   "AMR"    - for AMR
#   "iLBC"   - for ILBC
#-szBitRate - defines the bit rate for the codec being used. Depending on the codec the following values are valid:
#   G711Alaw    "64 kbps"
#   G711Ulaw    "64 kbps"
#   G723.1      "5.3 kbps", "6.3 kbps"
#   G726        "40 kbps"
#   G729A       "8 kbps"
#   G729B       "8 kbps"
#   AMR     "4.75 kbps", "5.15 kbps", "5.9 kbps", "6.7 kbps", "7.4 kbps", "7.95 kbps", "10.2 kbps", "12.2 kbps"
#   iLBC        "13.33 kbps", "15.2 kbps"
#-szCodecDetails - defines the codec details and has the following format:
#   BF<val1>PT<val2>
#   - <val1> represents number of codec bytes per frame
#   - <val2> represents the packet time
#  The possible values for <val1> and <val2> depend on codec type and bitrate
#
# Silence generation options:
#-bUseSilence - specifies if silence generation should be used (value 1) or not (value 0)
#-bSilenceMode - specifies the type of silence:
#   0 - Comfort Noise
#   1 - NULL data encoded using the configured codec
#-----------------------------------------------------------------------
$clientTraffic agentList(0).pm.mediaSettings.config \
    -szCodecName            "G711ALaw" \
    -szCodecDetails         "BF160PT20" \
    -bModifyPowerLevel      0 \
    -szPowerLevel           "PL_20" \
    -nJitterBuffer          1 \
    -bUseSilence            1 \
    -bSilenceMode           1 \
    -bRtpStartCollector     0

$clientTraffic agentList(0).pm.scenarios.appendItem \
     -id                    "ORIGINATECALL" \
     -symDestination        "198.18.0.101:5060"

$clientTraffic agentList(0).pm.scenarios.appendItem \
     -id                     "VOICESESSION" \
     -szAudioFile            "speech.wav" \
     -nPlayMode              1 \
     -nRepeatCount           1 \
     -nPlayTime              1 \
     -nTimeUnit              0

#-----------------------------------------------------------------------
# The GENERATEMF comand has the following possible parameters:
#-szMfSeq - the sequence of MF digits to be generated
#-nMfDuration - the duration of a MF digit; value range [10 990]
#-nInterMfInterval - the silence between the MF digits; value range [10 9990]
#-nMfAmplitude - the amplitude of the signal generated by the sending sequence; value range [-30 -10]
#-nPlayMode - the pley mode; possible values: 0 - generate fo a specified period of time
#                         1 - repeat for a specified number of times
#-nRepeatCount - number of times to repeat the generation of the sequence
#-nPlayTime - the time units to plays the specified sequence
#-nTimeUnit - time unit type; possible values: 0 - seconds
#                          1 - minutes
#                          2 - hours
#                          3 - days
#-----------------------------------------------------------------------
$clientTraffic agentList(0).pm.scenarios.appendItem \
     -id                    "GENERATEMF" \
     -szMfSeq               12345 \
     -nMfDuration           200 \
     -szCodecDetails        "BF160PT20" \
     -nInterMfInterval      40 \
     -nMfAmplitude          -10 \
     -nPlayMode             1 \
     -nRepeatCount          1 \
     -nPlayTime             30 \
     -nTimeUnit             0

#-----------------------------------------------------------------------
# The GENERATETONE comand has the following possible paramters:
#-nToneName - the id for the tone
#       possible values: 0 - "600-10"
#                    1 - "1400-10"
#                2 - "2500-10"
#                3 - "550-20"
#                4 - "1350-20"
#                5 - "2450-20"
#                6 - "650-30"
#                7 - "2550-30"
#                8 - "1450-30"
#                9 - "3400-10"
#                10 - "3400-30"
#                11 - "2100-10"
#                12 - "2150-30"
#                13 - "400-10"
#                14 - "450-30"
#                15 - "Confirmation Tone"
#                16 - "Call Waiting Tone"
#                17 - "TN_1"
#                -1 -"Custom Tone"
#
#-nPlayMode - the pley mode; possible values: 0 - generate fo a specified period of time
#                         1 - repeat for a specified number of times
#-nRepeatCount - number of times to repeat the generation of the sequence
#-nPlayTime - the time units to plays the specified sequence
#-nTimeUnit - time unit type; possible values: 0 - seconds
#                          1 - minutes
#                          2 - hours
#                          3 - days
#-nToneDuration - the duration of a tone with only one frequency
#-nFrequency1 - first frequency
#-nFrequency2 - second frequency
#-nAmplitude1 - first amplitude
#-nAmplitude2 - second amplitude
#-nOnTime - nOnTime must be equal to nToneDuration
#-nOffTime - the duration of silence 
#-nRepetitionCount - the number of repetition for the specified tone
#-nToneType - the tone type; possible values:   0 -"Single Tone"
#                       1 - "Dual Tone"
#                       2 - "Single Tone Cadence"
#                       3 - "Dual Tone Cadence"
#-----------------------------------------------------------------------
$clientTraffic agentList(0).pm.scenarios.appendItem \
     -id                    "GENERATETONE" \
     -nToneName             0 \
     -nToneDuration         100 \
     -nFrequency1           615 \
     -nFrequency2           0 \
     -nAmplitude1           -10 \
     -nAmplitude2           0 \
     -nOnTime               100 \
     -nOffTime              0 \
     -nRepetitionCount      1 \
     -nToneType             0 \
     -nPlayMode             1 \
     -nRepeatCount          1 \
     -nPlayTime             30 \
     -nTimeUnit             0

$clientTraffic agentList(0).pm.scenarios.appendItem \
     -id                    "ENDCALL"

#-----------------------------------------------------------------------
# SIP Server Configuration.
#-----------------------------------------------------------------------
set serverTraffic [::IxLoad new ixServerTraffic -name "ServerTraffic"]

$serverTraffic agentList.appendItem \
    -name                   "SIP_SERVER" \
    -protocol               "SIP" \
    -type                   "Server"

$serverTraffic agentList(0).pm.generalSettings.config \
     -szAuthUsername        "user" \
     -szAuthPassword        "password" \
     -szAuthDomain          "domain" \
     -szTransport           "UDP" \
     -nUdpPort              5060 \
     -nTcpPort              5060 \
     -nUdpMaxSize           1024 \
     -szRegistrar           "127.0.0.1:5060" \
     -bRegBefore            0

$serverTraffic agentList(0).pm.contentOfMessages.config \
     -bCompact              0 \
     -bFolding              0 \
     -bScattered            0 \
     -bAdvisable            0 \
     -bOptional             0 \
     -bBestPerformance      1 \
     -szREQUESTURI          "sip:IP" \
     -szFROM                "sip:id@IP" \
     -szTO                  "sip:id@IP" \
     -szCONTACT             "sip:id@IP"

$serverTraffic agentList(0).pm.scenarios.appendItem \
     -id                    "RECEIVEUSING180"


#-----------------------------------------------------------------------
# Create a client and server mapping and bind into the
# network and traffic that they will be employing.
#-----------------------------------------------------------------------
set clientTrafficNetworkMapping [::IxLoad new ixClientTrafficNetworkMapping \
    -network                $clientNetwork \
    -traffic                $clientTraffic \
    -objectiveType          $::ixObjective(kObjectiveTypeSimulatedUsers) \
    -objectiveValue         1 \
    -standbyTime            30 \
    -rampUpValue            1 \
    -sustainTime            40 \
    -rampDownTime           20
]

set serverTrafficNetworkMapping [::IxLoad new ixServerTrafficNetworkMapping \
    -network                $serverNetwork \
    -traffic                $serverTraffic \
    -matchClientTotalTime   1
]


#-----------------------------------------------------------------------
# Create the test and bind in the network-traffic mapping it is going
# to employ.
#-----------------------------------------------------------------------
set test [::IxLoad new ixTest \
    -name                   "Test" \
    -statsRequired          1 \
    -enableResetPorts       1 \
    -enableForceOwnership   1 \
]

$test clientCommunityList.appendItem -object $clientTrafficNetworkMapping

$test serverCommunityList.appendItem -object $serverTrafficNetworkMapping


#-----------------------------------------------------------------------
# Create a test controller bound to the previosuly allocated
# chassis chain. This will eventually run the test we created earlier.
#-----------------------------------------------------------------------
set testController [::IxLoad new ixTestController -outputDir 1]

$testController setResultDir "[pwd]/RESULTS/$logName"

$testController run $test -autorepository "test.rxf"

#-----------------------------------------------------------------------
# Have the script (v)wait until the test is over.
#-----------------------------------------------------------------------
vwait ::ixTestControllerMonitor
puts $::ixTestControllerMonitor


#-----------------------------------------------------------------------
# Cleanup.
#-----------------------------------------------------------------------
$testController releaseConfigWaitFinish

::IxLoad delete $chassisChain

::IxLoad delete $clientNetwork

::IxLoad delete $serverNetwork

::IxLoad delete $clientTraffic

::IxLoad delete $serverTraffic

::IxLoad delete $Filter_1

::IxLoad delete $Filter_2

::IxLoad delete $GratARP_1

::IxLoad delete $GratARP_2

::IxLoad delete $TCP_1

::IxLoad delete $TCP_2

::IxLoad delete $DNS_1

::IxLoad delete $DNS_2

::IxLoad delete $Settings_1

::IxLoad delete $Settings_2

::IxLoad delete $Ethernet_1

::IxLoad delete $Ethernet_2

::IxLoad delete $my_ixNetEthernetELMPlugin_1

::IxLoad delete $my_ixNetEthernetELMPlugin_2

::IxLoad delete $MAC_VLAN_1

::IxLoad delete $MAC_VLAN_2

::IxLoad delete $IP_1

::IxLoad delete $IP_2

::IxLoad delete $IP_R1

::IxLoad delete $IP_R2

::IxLoad delete $MAC_R1

::IxLoad delete $MAC_R2

::IxLoad delete $VLAN_R1

::IxLoad delete $VLAN_R2

::IxLoad delete $clientTrafficNetworkMapping

::IxLoad delete $serverTrafficNetworkMapping

::IxLoad delete $test

::IxLoad delete $testController

::IxLoad delete $logger

::IxLoad delete $logEngine


#-----------------------------------------------------------------------
# Disconnect / Release application lock.
#-----------------------------------------------------------------------

}] {
    puts $errorInfo
}

::IxLoad disconnect

