#-----------------------------------------------------------------------
# Setup path and load IxLoad package.
#-----------------------------------------------------------------------
source ../setup_simple.tcl


#-----------------------------------------------------------------------
# Initialize IxLoad
# IxLoad connect should always be called, even for local scripts.
#-----------------------------------------------------------------------
::IxLoad connect $::IxLoadPrivate::SimpleSettings::remoteServer

# Once we've connected, make sure we disconnect, even if there's a problem
if [catch {


#-----------------------------------------------------------------------
# Loads plugins for specific protocols configured in this test.
#-----------------------------------------------------------------------
global ixAppPluginManager
$ixAppPluginManager load "HTTP"


#-----------------------------------------------------------------------
# Logger setup.
#-----------------------------------------------------------------------
set logtag "IxLoad-api"
set logName "HTTP_L2L3Stats"
set logger [::IxLoad new ixLogger $logtag 1]
set logEngine [$logger getEngine]
$logEngine setLevels $::ixLogger(kLevelDebug) $::ixLogger(kLevelInfo)
$logEngine setFile $logName 2 256 1


#-----------------------------------------------------------------------
# Package require the stat collection utilities.
#-----------------------------------------------------------------------
package require statCollectorUtils
set scu_version [package require statCollectorUtils]
puts "statCollectorUtils package version = $scu_version"


#-----------------------------------------------------------------------
# Build Chassis Chain.
#-----------------------------------------------------------------------
set chassisChain [::IxLoad new ixChassisChain]

$chassisChain addChassis $::IxLoadPrivate::SimpleSettings::chassisName


#-----------------------------------------------------------------------
# Build Client Network.
#-----------------------------------------------------------------------
set clientNetwork [::IxLoad new ixNetworkGroup $chassisChain]

$clientNetwork config \
	-name 					"ClientNetwork"

$clientNetwork networkRangeList.appendItem \
    -name           		"ClientRange" \
    -enable         		1 \
    -firstIp        		"198.18.2.1" \
    -ipIncrStep     		$::ixNetworkRange(kIpIncrOctetForth) \
    -ipCount        		100 \
    -networkMask    		"255.255.0.0" \
    -gateway        		"0.0.0.0" \
    -firstMac       		"00:C6:12:02:01:00" \
    -macIncrStep    		$::ixNetworkRange(kMacIncrOctetSixth) \
    -vlanEnable     		0 \
    -vlanId         		1 \
    -mssEnable      		0 \
    -mss            		100

$clientNetwork arpSettings.config -gratuitousArp 0


#-----------------------------------------------------------------------
# Assign Ports to Client Network.
#-----------------------------------------------------------------------
$clientNetwork portList.appendItem \
    -chassisId  			1 \
    -cardId     			$::IxLoadPrivate::SimpleSettings::clientPort(CARD_ID)\
    -portId     			$::IxLoadPrivate::SimpleSettings::clientPort(PORT_ID)


#-----------------------------------------------------------------------
# Build Server Network.
#-----------------------------------------------------------------------
set serverNetwork [::IxLoad new ixNetworkGroup $chassisChain]

$serverNetwork config -name "ServerNetwork"

$serverNetwork networkRangeList.appendItem \
    -name           		"ServerRange" \
    -enable         		1 \
    -firstIp        		"198.18.200.1" \
    -ipIncrStep     		$::ixNetworkRange(kIpIncrOctetForth) \
    -ipCount        		1 \
    -networkMask    		"255.255.0.0" \
    -gateway        		"0.0.0.0"\
    -firstMac       		"00:C6:12:02:02:00" \
    -macIncrStep    		$::ixNetworkRange(kMacIncrOctetSixth) \
    -vlanEnable     		0 \
    -vlanId         		1 \
    -mssEnable      		0 \
    -mss            		100

$serverNetwork arpSettings.config -gratuitousArp 0

#-----------------------------------------------------------------------
# Assign Ports to Server Network.
#-----------------------------------------------------------------------
$serverNetwork portList.appendItem \
    -chassisId  			1 \
    -cardId     			$::IxLoadPrivate::SimpleSettings::serverPort(CARD_ID)\
    -portId     			$::IxLoadPrivate::SimpleSettings::serverPort(PORT_ID)


#-----------------------------------------------------------------------
# Construct Client Traffic
# The ActivityModel acts as a factory for creating agents which actually
# generate the test traffic.
#-----------------------------------------------------------------------
set clientTraffic [::IxLoad new ixClientTraffic -name "ClientTraffic"]

$clientTraffic agentList.appendItem \
    -name                   "HTTPClient" \
    -protocol               "HTTP" \
    -type                   "Client" \
    -maxSessions            3 \
    -httpVersion            $::HTTP_Client(kHttpVersion10) \
    -keepAlive              0 \
    -maxPersistentRequests  3 \
    -followHttpRedirects    0 \
    -enableCookieSupport    0 \
    -enableHttpProxy        0 \
    -enableHttpsProxy       0 \
    -browserEmulation       $::HTTP_Client(kBrowserTypeIE5) \
    -enableSsl              0  
                    
					
#-----------------------------------------------------------------------
# Add actions to this client agent.
#-----------------------------------------------------------------------
foreach {pageObject destination} {
    "/4k.htm" "ServerTraffic_HTTPServer"
    "/8k.htm" "ServerTraffic_HTTPServer"
    "/128k.htm" "ServerTraffic_HTTPServer"
} {
    $clientTraffic agentList(0).actionList.appendItem  \
        -command        	"GET" \
        -destination    	$destination \
        -pageObject     	$pageObject
}


#-----------------------------------------------------------------------
# Construct Server Traffic.
#-----------------------------------------------------------------------
set serverTraffic [::IxLoad new ixServerTraffic -name "ServerTraffic"]

$serverTraffic agentList.appendItem \
    -name       			"HTTPServer" \
    -protocol   			"HTTP" \
    -type       			"Server" \
    -httpPort   			80 

for {set idx 0} {$idx < [$serverTraffic agentList(0).responseHeaderList.indexCount]} {incr idx} {
    set response [$serverTraffic agentList(0).responseHeaderList.getItem $idx]
    if {[$response cget -name] == "200_OK"} {
        set response200ok $response
    }
    if {[$response cget -name] == "404_PageNotFound"} {
        set response404_PageNotFound $response
    }
}

#-----------------------------------------------------------------------
# Clear pre-defined web pages, add new web pages.
#-----------------------------------------------------------------------
$serverTraffic agentList(0).webPageList.clear

$serverTraffic agentList(0).webPageList.appendItem \
    -page           		"/4k.html" \
    -payloadType    		"range" \
    -payloadSize    		"4096-4096" \
    -response       		$response200ok

$serverTraffic agentList(0).webPageList.appendItem \
    -page           		"/8k.html" \
    -payloadType    		"range" \
    -payloadSize    		"8192-8192" \
    -response       		$response404_PageNotFound

$serverTraffic agentList(0).webPageList.appendItem \
    -page           		"/128k.html" \
    -payloadType    		"range" \
    -payloadSize    		"131072" \
    -response       		$response200ok


#-----------------------------------------------------------------------
# Create a client and server mapping and bind into the
# network and traffic that they will be employing.
#-----------------------------------------------------------------------
set clientTrafficNetworkMapping [::IxLoad new ixClientTrafficNetworkMapping \
    -network                $clientNetwork \
    -traffic                $clientTraffic \
    -objectiveType          $::ixObjective(kObjectiveTypeSimulatedUsers) \
    -objectiveValue         20 \
    -rampUpValue            5 \
    -sustainTime            20 \
    -rampDownTime           20
]

set serverTrafficNetworkMapping [::IxLoad new ixServerTrafficNetworkMapping \
    -network                $serverNetwork \
    -traffic                $serverTraffic \
    -matchClientTotalTime   1
]


#-----------------------------------------------------------------------
# Set objective type, value and port map for client activity.
#-----------------------------------------------------------------------
$clientTrafficNetworkMapping setObjectiveTypeForActivity "HTTPClient" $::ixObjective(kObjectiveTypeConnectionRate)

$clientTrafficNetworkMapping setObjectiveValueForActivity "HTTPClient" 200

$clientTrafficNetworkMapping setPortMapForActivity "HTTPClient" $::ixPortMap(kPortMapFullMesh)


#-----------------------------------------------------------------------
# Create the test and bind in the network-traffic mapping it is going
# to employ.
#-----------------------------------------------------------------------
set test [::IxLoad new ixTest \
    -name               	"Test" \
    -statsRequired      	1 \
    -enableResetPorts   	0
]

$test clientCommunityList.appendItem -object $clientTrafficNetworkMapping

$test serverCommunityList.appendItem -object $serverTrafficNetworkMapping


#-----------------------------------------------------------------------
# Create a test controller bound to the previosly allocated
# chassis chain. This will eventually run the test we created earlier.
#-----------------------------------------------------------------------
set testController [::IxLoad new ixTestController -outputDir 1]

$testController setResultDir "RESULTS/$logName"


#-----------------------------------------------------------------------
# Set up stat collection.
#-----------------------------------------------------------------------
set NS statCollectorUtils

set ::test_server_handle [$testController getTestServerHandle]

${NS}::Initialize -testServerHandle $::test_server_handle

#-----------------------------------------------------------------------
# Clear any stats that may have been registered previously.
#-----------------------------------------------------------------------
${NS}::ClearStats

#-----------------------------------------------------------------------
# Define the L2L3 stats we would like to collect.
#-----------------------------------------------------------------------
${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_1" \
    -statSourceType "PortMonitor" \
    -statName "Link State" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_2" \
    -statSourceType "PortMonitor" \
    -statName "Line Speed" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_3" \
    -statSourceType "PortMonitor" \
    -statName "Frames Sent" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_4" \
    -statSourceType "PortMonitor" \
    -statName "Frames Sent Rate" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_5" \
    -statSourceType "PortMonitor" \
    -statName "Valid Frames Received" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_6" \
    -statSourceType "PortMonitor" \
    -statName "Valid Frames Received Rate" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_7" \
    -statSourceType "PortMonitor" \
    -statName "Bytes Sent" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_8" \
    -statSourceType "PortMonitor" \
    -statName "Bytes Sent Rate" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_9" \
    -statSourceType "PortMonitor" \
    -statName "Bytes Received" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_10" \
    -statSourceType "PortMonitor" \
    -statName "Bytes Received Rate" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_11" \
    -statSourceType "PortMonitor" \
    -statName "Transmit Arp Request" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_12" \
    -statSourceType "PortMonitor" \
    -statName "Transmit Arp Reply" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_13" \
    -statSourceType "PortMonitor" \
    -statName "Receive Arp Request" \
    -aggregationType kNone \
    -filterList {}

${NS}::AddL2L3Stat \
    -caption "Watch_Stat_L2L3_14" \
    -statSourceType "PortMonitor" \
    -statName "Receive Arp Reply" \
    -aggregationType kNone \
    -filterList {}

#-----------------------------------------------------------------------
# Start the collector (runs in the tcl event loop).
#-----------------------------------------------------------------------
proc ::my_stat_collector_command {args} {
    puts "====================================="
    puts "INCOMING STAT RECORD >>> $args"
    puts "Len = [llength $args]"
    puts  [lindex $args 0]
    puts  [lindex $args 1]
    puts "====================================="
}

${NS}::StartCollector -command ::my_stat_collector_command

$testController run $test


#-----------------------------------------------------------------------
# Have the script (v)wait until the test is over.
#-----------------------------------------------------------------------
vwait ::ixTestControllerMonitor
puts $::ixTestControllerMonitor


#-----------------------------------------------------------------------
# Stop the collector (running in the tcl event loop).
#-----------------------------------------------------------------------
${NS}::StopCollector


#-----------------------------------------------------------------------
# Cleanup.
#-----------------------------------------------------------------------
$testController releaseConfigWaitFinish

::IxLoad delete $chassisChain

::IxLoad delete $clientNetwork

::IxLoad delete $serverNetwork

::IxLoad delete $clientTraffic

::IxLoad delete $serverTraffic

::IxLoad delete $clientTrafficNetworkMapping

::IxLoad delete $serverTrafficNetworkMapping

::IxLoad delete $test

::IxLoad delete $testController

::IxLoad delete $logger

::IxLoad delete $logEngine


#-----------------------------------------------------------------------
# Disconnect / Release application lock.
#-----------------------------------------------------------------------

}] {
    puts $errorInfo
}

::IxLoad disconnect
