#
# IxLoad.pm
#
use 5.010;
use Tcl;
use warnings;
use strict;
use Carp;
use threads;
use XML::Parser;
use File::Basename;
my($operatingSystem) = "$^O\n";


package IxLoadTcl;
sub initialize_package {
    #
    # make tcl interpreter used to do all the work
    #
    $IxLoadTcl::interp = new Tcl;
    $IxLoadTcl::interp->Init();
    if($operatingSystem eq "linux\n")
    {
	$ENV{PERL_DL_NONLAZY} =1;
	$IxLoadTcl::interp->Eval('
        if {[info commands ::perl::Eval] != ""} {
            set orig_env [array get ::env]
            array unset ::env
            
            array set ::env $orig_env

            proc ::perl::env_writeback_hack {varName index op} {
                    ::perl::Eval "\$ENV\{\'$index\'\} = \'$::env($index)\'"
            }
            proc ::perl::env_readback_hack {varName index op} {
                    if {[::perl::Eval "exists(\$ENV{\'$index\'})"]} {
                            set ::env($index) [::perl::Eval "\$ENV{\'$index\'}"]
                    }
            }
            trace add variable ::env write ::perl::env_writeback_hack
            trace add variable ::env read ::perl::env_readback_hack
        }
    ');
        foreach(@INC){
        $IxLoadTcl::interp->invoke("set", "plPath", $_);
        $IxLoadTcl::interp->Eval(q(set auto_path [linsert $auto_path 0 $plPath]));  
        }
    }
    #
    # Util for flattening a hash into a tcl option value list
    # {-option0 value0 ... -optionN valueN} list
    #
    # Note: not for general consumption subject to change 
    # w/o notice
    #
    sub _flatten {
		my $noOfArguments = scalar @_;
		
		my @args = ();
		for (my $i=0; $i < $noOfArguments; $i++)
		{
			my $arg = $_[$i];
			
			if (ref($arg) eq 'HASH') {
				my @l = ();
				while ((my $k, my $v) = each(%$arg)) {
					$k =~ s/^\-//;
					push(@l,"-".$k);
					push(@l,$v);
				}
				push(@args,@l);
			}
			else
			{
				if (ref($arg) eq 'ARRAY') {
					push(@args,@$arg);
				}
				else{
					push(@args, $arg);
				}
			}
		}
		
        return @args;
    }

    #
    # I am not entirely trustful of ties between perl and tcl
    # so just supply an accessor to errorInfo instead of using a tie
    sub errorInfo {
        $::IxLoadTcl::interp->invoke('set', '::errorInfo');
    };

    #
    # Make a set of bindings for the basic tcl commands
    #
    for my $cmd (
    "after",
    "append",
    "array",
    "bgerror",
    "binary",
    "break",
    "catch",
    "cd",
    "clock",
    "close",
    "concat",
    "continue",
    "dde",
    "encoding",
    "eof",
    "error",
    "eval",
    "exec",
    "exit",
    "expr",
    "fblocked",
    "fconfigure",
    "fcopy",
    "file",
    "fileevent",
    "filename",
    "flush",
    "for",
    "foreach",
    "format",
    "gets",
    "glob",
    "global",
    "if",
    "incr",
    "info",
    "interp",
    "join",
    "lappend",
    "lindex",
    "linsert",
    "list",
    "llength",
    "load",
    "lrange",
    "lreplace",
    "lsearch",
    "lset",
    "lsort",
    "namespace",
    "open",
    "package",
    "parray",
    "pid",
    "proc",
    "puts",
    "pwd",
    "read",
    "regexp",
    "registry",
    "regsub",
    "rename",
    "resource",
    "return",
    "scan",
    "seek",
    "set",
    "socket",
    "source",
    "split",
    "string",
    "subst",
    "switch",
    "tell",
    "time",
    "trace",
    "unknown",
    "unset",
    "update",
    "uplevel",
    "upvar",
    "variable",
    "vwait",
    "while"
    ) {
        my $ss = 
          "sub $cmd " . '{ $IxLoadTcl::interp->invoke('. "\"$cmd\"". ', @_); }';
        #print("<" . $ss . ">\n");
        eval($ss);
        #print("done\n");
    }

    # Make a few dummy commands related to the tk that the hlt_init.tcl file
    # (and other similar ixia XXX_init.tcl files)
    # try to use to hide/show windows etc..
    #$IxLoadTcl::interp->invoke("proc", "console", 'args', '');
    #$IxLoadTcl::interp->invoke("proc", "wm", 'args', '');
    #$IxLoadTcl::interp->invoke("set", "::tcl_interactive", "0");

    print("IxLoadTcl: Tcl version: " . IxLoadTcl::info("patchlevel"). "\n");
    $IxLoadTcl::initialized = IxLoadTcl::info("patchlevel");
}

package IxLoad;
sub initialize_package {
	$::IxLoad::connect = shift @_;#get the connection string.
    $::IxLoad::_dbg_flag = 0;
    $::IxLoad::_trace_tcl_calls = 0;
	
    sub _dbg {
        my $s = shift;
        if ($::IxLoad::_dbg_flag == 1) {
            print('IxLoad.pm: '.$s);
        }
    }
    sub traceTclCalls {
        my $val=shift;
        $::IxLoad::_trace_tcl_calls=$val;
    }
    sub debugPerlModule {
        my $val = shift;
        $::IxLoad::_dbg_flag=$val;
        $::IxLoad::_trace_tcl_calls=$val;
    }
    #-------
    # The init file used for ixn
	if($operatingSystem ne "linux\n") {
        use Cwd;
        my $absPath = Cwd::abs_path(__FILE__);
        
        my ($volume, $directory, $file) = File::Spec->splitpath($absPath);
        my $currentModulePath = $volume.$directory;

		my @values = split('/', $currentModulePath);
		my $pathToRootFolder = "";
		
		foreach my $val (@values) {
			if ($val ne "PerlScripts") {
				$pathToRootFolder .= $val;
				$pathToRootFolder .= '/';
			}
			else {
				last;
			}
		}

		#the path to the root folder contains : "D:/ixia/ixload 6.60.0.408-eb/". Just append TclScripts folder hierarchy.
		my $ixiaWishPath = $pathToRootFolder."TclScripts/bin/IxiaWish.tcl";
		
		my @l = IxLoadTcl::glob($ixiaWishPath);
		
		$ixiaixn::ixninitfile = $l[-1];
		#-------
		
		#--------
		# - Source the ixn init file
		# - Package require the underlying IxLoad tcl package
		#
		eval { 
			$IxLoadTcl::interp->invoke("source", $ixiaixn::ixninitfile); 
		};
	}
    if ($@) {
        print(
            "Error sourcing tcl file \"" . 
            $ixiaixn::ixninitfile .
            "\":\n---------\n" . 
            IxLoadTcl::errorInfo() . 
            "\n----------\n\n"
        );
    }
	
	#don't initialize the pipe for linux
	if($operatingSystem ne "linux\n") {
		initializePipe();
	}

	#--------
    # Static commands
    #
    for my $cmd (
        "connect",
        "disconnect",

        "debuglevel"
    ) {
        my $ss = 
          "sub $cmd " . '{my $class=shift;$IxLoadTcl::interp->invoke('. "'::IxLoad', ". "'$cmd'". ', IxLoad::_tcl_args(@_)); }';
        #_dbg("<" . $ss . ">\n");
        eval($ss);
        #_dbg("done\n");
    }
    #--------
	
    _dbg(
        'IxLoad version: '.
        IxLoadTcl::package('require', 'IxLoadCsv')."\n"
    );
	
	IxLoad->connect($::IxLoad::connect);
	
    _dbg(
        'IxLoad statcollectorUtils: '.
        IxLoadTcl::package('require', 'statCollectorUtils')."\n"
    );
	
    #--------


    #--------
    # IxLoad class defns
    #--------

    sub craft {
        my $class = shift;
        my $self = {};
        $self->{_tcltype} = shift;
        $self->{_handle} = shift;
        $self->{_subhandles} = {};
        _dbg("craft: type = $self->{_tcltype} handle = ".$self->{_handle}."\n");
        bless($self, $class);
        return $self;
    }

    # Prepare args for a call into IxLoad tcl land. This involves the 
    # following:
    #
    # 1) replace and IxLoad objects w/ the tcl handle it 
    # contains within
    #
    sub _tcl_args {
        my @r = (); foreach my $i (@_) {
            my $type = ref($i) or do {
                push(@r, $i);
                next;
            };
            if ($type eq "IxLoad") {
                push(@r, $i->_get_tcl_handle());
            } else {
                push(@r, $i);
            }
        }
        return @r;
    }
    sub _get_tcl_handle {
        my $self = shift;
        return $self->{_handle};
    }
    sub _logtime {
        my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(
        );
        ##
        ## Values, in order, left to right, are:
        ##
        ## $sec - seconds [0-60, to allow for leap seconds]
        ## $min - minutes [0-59],
        ## $hour - hours [0-23],
        ## $mday - day of the month [1-31] (Note: from '1'),
        ## $mon - month [0-11] (Jan. is '0'),
        ## $year - number of years since 1900 (1999 is 99; 2000 is 100),
        ## $wday - day of the week [0-6] (Sun. is '0'),
        ## $yday - day of the year [0-365] (total of 366 possible for leap year)
        ## $isdst - daylight savings time [0,1] -- 
        ##          returns boolean '0' or '1' if Daylight Savings active.
        return "$hour:$min:$sec";
    }
    sub new {
        my $class = shift;
        my $self = {};
        $self->{_tcltype} = shift;
        $self->{_subhandles} = {};
        #
        # for items in @_
        # we need to converting perl object wrappers
        # back into tcl object names for items
        #
        @_ = IxLoad::_tcl_args(IxLoadTcl::_flatten(@_));
        if ($::IxLoad::_trace_tcl_calls == 1) {
            my $c = (caller(0))[1]."(".(caller(0))[2]."): ";
            #print(
            #    IxLoad::_logtime().
            #    ':CALL '.$c."::IxLoad new ".$self->{_tcltype}." ".
            #    join(' ',@_)."\n"
            #);
        }
        $self->{_handle} = $IxLoadTcl::interp->invoke(
            '::IxLoad', 'new', $self->{_tcltype}, @_
        );
        _dbg("new: type = $self->{_tcltype} handle $self->{_handle}\n");
        bless($self, $class);
        return $self;
    }
	
	sub delete {
        my $self = shift;
		my $object = shift;
        
        $IxLoadTcl::interp->invoke('eval', '::IxLoad', 'delete', $object->{_handle});
        _dbg("deleted TCL handle = $object->{_handle}\n");
    };
	
	
    sub AUTOLOAD {
        my $self = shift;
        my $type = ref($self) or do {Carp::confess("$self is not an object");};
        my $tail = $IxLoad::AUTOLOAD;
        $tail =~ s/.*://;   # strip fully-qualified portion
		
        @_ = IxLoadTcl::_flatten(@_);
		
        if ($::IxLoad::_trace_tcl_calls == 1) {
            my $c = (caller(0))[1]."(".(caller(0))[2]."): ";
            print(
                IxLoad::_logtime().
                ':CALL '.$c.
                $self->{_handle}." ".
                $self->{_tcltype}.".".$tail." ".
                join(' ',@_)."\n"
            );
        }

		my @args = formatArgs($tail, @_);
		
        my $rval=$IxLoadTcl::interp->invoke(
			"eval",
            $self->{_handle},
            $tail, 
            ::IxLoad::_tcl_args(@args)
        );
		
        if ($rval =~ m/^::tp::_Obj.*/) {
            # resulted in a tcl object, craft a perl object wrapper for it
            _dbg("crafting object for $rval...\n");
            $rval = IxLoad->craft($self->{_tcltype}.".".$tail, $rval);
        }
        _dbg("RETURN: <".$rval."> errrorInfo\n".IxLoadTcl::errorInfo()."\n");
	
        return $rval;
    };

	#if the attribute to add is a string, escape it
	#example: don't pass Mike, pass "Mike"
	sub getAttributeToAdd{
		use Scalar::Util;
		my $arg = shift;
		my $returnArgument = "";
		
		#only escape the string if it doesn't look like a number, and not an object reference
		if ((Scalar::Util::looks_like_number($arg) == 0) && (ref($arg) eq "")) {
			$returnArgument = "\"$arg\"";
		}
		else {
			$returnArgument = $arg;
		}
		
		return $returnArgument;
	}
	
	sub escapeBrackets{
        my $value = shift;

        #only escape brackets if they're not already escaped
        if ( (index($value, '[') != -1) && (index($value, '\[') == -1) ) {
            $value =~ s/\[/\\\[/ig;    
        }
        
        if ( (index($value, ']') != -1) && (index($value, '\]') == -1) ) {
            $value =~ s/\]/\\\]/ig;
        }

        return $value;
    }
	
	sub escapeBackSlashes{
        my $value = shift;
        #don't escape the characters that already come escaped in the script!
        if ( (index($value, '$') == -1) && (index($value, '"') == -1) && (index($value, '{') == -1) && (index($value, '}') == -1) && (index($value, '[') == -1) && (index($value, ']') == -1) && (index($value, '\\') == -1)){
            $value =~ s/\\/\\\\/ig;
        }
        return $value;
	}
	
	sub formatArgs {
		my $tail = shift;
		
		my $arg;
		my @args = @_;
		my @formattedArgs = ();
		
		if ($tail eq "cget"){
			foreach $arg (@args){
				my $formattedArg = "-$arg";
				push(@formattedArgs, $formattedArg);
			}
		}
		else{
			@formattedArgs = @args;
		}
		
		@args = ();
		#don't just pass null strings to TCL, pass a string containing ""
		foreach $arg (@formattedArgs){
        
        
			my $argToAdd = escapeBackSlashes($arg);
			$argToAdd = escapeBrackets($argToAdd);#sometimes, brackets [] are not escaped. must escape them
			
            $argToAdd = getAttributeToAdd($argToAdd);
            
			push(@args, $argToAdd);
		}

		return @args;
	}

    sub pluginManager {
        my $class=shift;
		
		my $pluginManagerVar = '$ixAppPluginManager';
		$IxLoadTcl::interp->invoke(qw(global ixAppPluginManager));
		
		return($IxLoadTcl::interp->invoke('eval', '$ixAppPluginManager', @_));
    }

    sub TestControllerWait {
        IxLoadTcl::vwait('::ixTestControllerMonitor');
        return(IxLoadTcl::set('::ixTestControllerMonitor'));
    }
	
	sub waitForCaptureDataReceived {

		my $command = << 'END_COMMAND';
		if {$::ixCaptureMonitor == ""} {
			puts "Waiting for last capture data to arrive."
			vwait ::ixCaptureMonitor
			puts "Capture data received."
			}
END_COMMAND
		
		$IxLoadTcl::interp->invoke(
			"eval",
			"$command"
		);
		
	}

    sub createPipe {
        my $pid = $$;
        my $pipeName = "IxLoad$pid";

		require Win32::Pipe;
		
        my $pipe = new Win32::Pipe($pipeName);
        
        $pipe->Connect();

        while (1) {
            my $data = $pipe->Read();

            if (defined $data) {
                my $p1 = new XML::Parser(Handlers => {
                    Char => \&parseMessage
                    });

                $p1->parse($data);
            }
            sleep(0.5);
            
        }

    }

    sub parseMessage {
        my ($parserObj, $message) = @_;
        
        if (defined $message && $message eq "exit") {
            exit(2);
        }
    }

    sub initializePipe {
        my $thread = threads->new(\&createPipe, $_)->detach();

    }
};

package IxLoad::StatCollector;
sub initialize_package() {
    sub _callprint {
        my $cmd = shift;
        print(">>>>> ".qw(@_)."\n");
        print(
            IxLoad::_logtime().
            ":CALL: ".
            (caller(1))[1]."(".(caller(1))[2]."): ".
            $cmd." ".join(' ', @_).
            "\n"
        );
    }
    sub new {
        my $class = shift;
        my $self = {};

        bless($self, $class);
        return $self;
    }
	
	sub Initialize {
		my $handle = $_[1];
		
		$IxLoadTcl::interp->invoke(
			'eval',
            '::statCollectorUtils::Initialize',
            '-testServerHandle',
            "{::perl::Eval $handle}"
        );
		
		my @cmds = IxLoadTcl::info('commands',  '::statCollectorUtils::*');

        foreach my $cmd (@cmds) {
            my $tail = $cmd;
            $tail =~ s/.*://;   # strip fully-qualified portion
            if ($tail eq 'StartCollector') {
                next;
            }
            my $ss = 
            "sub $tail {\n".
                " my \$self=shift;\n".
                " _callprint('$cmd',".'IxLoadTcl::_flatten(@_));'."\n".
                ' $IxLoadTcl::interp->invoke('."\n".
                "    '$cmd',\n".
                '    IxLoadTcl::_flatten(@_)'."\n".
                " );\n".
            "};\n"
            ;
            print("<$ss>\n");
            eval $ss;
        }
	}
	
	sub ClearStats {
		$IxLoadTcl::interp->invoke(
			'eval',
            'statCollectorUtils::ClearStats',
        );
	}
	
    sub StartCollector {
        my $self = shift;
        @_ = IxLoadTcl::_flatten(@_);
        my $commandName = "registerStatCommand";
        my $showStatSources = "False";
        my $showStatNames = "False";
        
        # execute for loop three times, to parse all the possible parameters.
        my @a = (1..3);
        for my $i (@a){
            my $param = shift;
            
            if ($param eq "-command") {
                my $command = shift;
                $IxLoadTcl::interp->CreateCommand($commandName, $command);
                
                if ($command eq "") {
                    Carp::confess(
                        "bad syntax: _perl_command_ empty/not supplied: ".
                        " usage: -command _perl_command_");
                }
            }
            
            if ($param eq "-showStatSources"){
                my $paramValue = shift;
                if ($paramValue eq ""){
                    Carp::confess(
                    "bad syntax: _perl_showStatSources_ empty/not supplied: ".
                    " usage: -showStatSources _perl_showStatSources_");
                }
                $showStatSources = $paramValue;
            }
            if ($param eq "-showStatNames"){
                my $paramValue = shift;
                if ($paramValue eq ""){
                    Carp::confess(
                    "bad syntax: _perl_showStatNames_ empty/not supplied: ".
                    " usage: -showStatNames _perl_showStatNames_");
                }
                $showStatNames = $paramValue;
            }
        }
		
        _callprint(
            '::statCollectorUtils::StartCollector',
            '-command',
            "$commandName",
            '-showStatSources',
            "$showStatSources",
            '-showStatNames',
            "$showStatNames"
        );
		
		
		
        $IxLoadTcl::interp->invoke(
			'eval',
            '::statCollectorUtils::StartCollector',
            '-command',
            "$commandName",
            '-showStatSources',
            "$showStatSources",
            '-showStatNames',
            "$showStatNames"
        );
    }

    sub StopCollector {
		$IxLoadTcl::interp->invoke(
			'eval',
            'statCollectorUtils::StopCollector',
        );
	}

	sub formatStatArgs{
		my $argsDict = shift;
		my $formattedArgs = "";
		
		foreach (keys %{$argsDict})
		{
			my $key = $_;
			my $value = $argsDict->{$_};
			
			if ($key eq "filterList"){
				$formattedArgs .= " -$key {";
				
				foreach (keys %{$value}){
					my $filterName = $_;
					my @filters = $value->{$filterName};
					
					$formattedArgs .= "\"$filterName\" {";
					
					foreach (@filters) {
						my $filter = $_;
						$formattedArgs .= "\"$filter\"";
					}
					
					$formattedArgs .= "} ";
				}
				
				$formattedArgs .= "} ";
			}
			else {
				$value = IxLoad::escapeBackSlashes($value);
                $value = IxLoad::escapeBrackets($value);#sometimes, brackets [] are not escaped. must escape them
				
                $value = IxLoad::getAttributeToAdd($value);
                
				$formattedArgs .= " -$key $value";
			}
		
		}
		
		return $formattedArgs;
	}
	
	sub AddStat {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddStat',
            $formattedTclArgs,
        );
	}
	
	sub AddNetworkStat {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddNetworkStat',
            $formattedTclArgs,
        );
	}
	
	sub AddL2L3Stat {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddL2L3Stat',
            $formattedTclArgs,
        );
	}

	sub AddPerInterfaceStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddPerInterfaceStats',
            $formattedTclArgs,
        );
	}

	sub AddPerUrlPerIpStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddPerUrlPerIpStats',
            $formattedTclArgs,
        );
	}	

	sub AddSIPPerStreamStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddSIPPerStreamStats',
            $formattedTclArgs,
        );
	}	
	
	sub AddMGCPPerStreamStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddMGCPPerStreamStats',
            $formattedTclArgs,
        );
	}	
	
	sub AddVideoPerStreamStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddVideoPerStreamStats',
            $formattedTclArgs,
        );
	}	

   	sub AddVoIPPerChannelStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddVoIPPerChannelStats',
            $formattedTclArgs,
        );
	}	

   	sub AddVoIPSipPerChannelStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddVoIPSipPerChannelStats',
            $formattedTclArgs,
        );
	}	
	
   	sub AddVoIPSkinnyPerChannelStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddVoIPSkinnyPerChannelStats',
            $formattedTclArgs,
        );
	}	

   	sub AddH248TermGroupPerChannelStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddH248TermGroupPerChannelStats',
            $formattedTclArgs,
        );
	}	

	sub AddVoIPH323PerChannelStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddVoIPH323PerChannelStats',
            $formattedTclArgs,
        );
	}

	sub AddSIPPerVideoCallStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddSIPPerVideoCallStats',
            $formattedTclArgs,
        );
	}

	sub AddVideoIPTVPerStreamStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddVideoIPTVPerStreamStats',
            $formattedTclArgs,
        );
	}

	sub AddPerStreamStats {
		my @args = ();
		
		my $test = shift;
		my $argsDict = shift;
		my $formattedTclArgs = formatStatArgs($argsDict);
		
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::AddPerStreamStats',
            $formattedTclArgs,
        );
	}

	sub GetRegisterStatsCommandXml {
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::GetRegisterStatsCommandXml'
        );
	}

    sub SetCsvVersion {
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::SetCsvVersion'
        );
	}

	sub SetCsvThroughputUnits {
		$IxLoadTcl::interp->invoke(
            'eval',
            '::statCollectorUtils::SetCsvThroughputUnits'
        );
	}
	
    
    sub DESTROY {
        my $self = shift;
    };
};

package IxLoad::Info;
sub initialize_package() {
    #
    # re-bind any existing tcl->python bindings in place yet again into perl
    # so we that we end up w/ 
    #
    # perl->tcl-python 
    #
    # variable bindings. (insert eye rolling here..)
    #
	my @bv_list = $IxLoadTcl::interp->invoke(qw(::IxLoad bindvariable));
	while (my ($tcl_name, $bvdata) = splice(@bv_list, 0, 2)) {
        my $cmd = "";
        my $perl_name = $tcl_name;
        $perl_name =~ s/^[:]*//;   # strip leading colons if any
        $perl_name = "IxLoad::Info::".$perl_name;
        if (IxLoadTcl::array('exists', $tcl_name) ne 1) {
            $cmd = 'tie $';
        } else {
            $cmd = 'tie %'; #underlying var is an array, bind it to a perl hash
        }
        $cmd = $cmd
            ."$perl_name, 'Tcl::Var', "
            .'$IxLoadTcl::interp, '
            ."'$tcl_name';";
        #print("<<$cmd>>\n");
        eval $cmd or do {Carp::confess("tie command error");}
    }
};

#
#
#

package IxLoadConnect;
sub initialize_package(){
	sub connect{
		$IxLoadConnect::conn = $_[1];#retrieve the connection string (ex "1.2.3.4")
		IxLoadTcl::initialize_package();
		IxLoad::initialize_package($IxLoadConnect::conn);
		IxLoad::Info::initialize_package();
		IxLoad::StatCollector::initialize_package();
		IxLoadTcl::set('::errorInfo', '');
	}
}

IxLoadConnect::initialize_package();
1
